/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package base;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.List;

import main.FunSiPJar;

import org.apache.log4j.Logger;

import util.DualLocationList;
import util.FastaFileData;
import util.RocCurveData;
import util.classificationFeatures.ClassificationFeature;
import util.classificationFeatures.ClassificationFeatureUtils;
import util.classificationFeatures.FeatureExtractionException;
import util.classificationFeatures.SeconderyStructureClassificationFeature;
import util.conversions.Conversion;
import util.featureSelection.FeatureSelectionStub;
import util.gene.DNAUtil;
import util.io.CrossValidationOutput;
import util.io.FileParser;
import util.io.IOUtils;
import util.results.CrossValidationResult;
import util.secondaryStructure.SecondaryStructureData;
import util.secondaryStructure.SecondaryStructureExtraction;
import classifier.Classifier;
import classifier.LibSVM_Weka;
import classifier.SMO_Weka;
import classifier.SVMOptions;

/**
 * This class represents the dataflow that is necessary to either build 
 * a classification model for a certain functional site, or evaluate a 
 * sequence for said functional site. 
 * 
 * This class is thus part of the backbone of the FunSiP-program, combining
 * all the available resources and classes to produce valid data.
 * 
 * Failure of execution in this class will return the workflow-pointer to 
 * the SprWorkflow-object, thus ending the objectives set for this classification
 * action. In other words, the execution of ClassificationAction is as
 * atomic as possible.
 * 
 * @author Michiel Van Bel
 *
 */
public class ClassificationAction implements Runnable, Cloneable{
	
	//variables used for testing and debugging for the program itself.
	//!! variables used for testing how fast features are extracted and evaluated !!//
	private boolean debug_speed = false;		
	private long d_s_extractionTime = 0;  		
	private long d_s_evalTime = 0;		
	
	
	/**
	 * The different dataflow/workflow commands. Either a new model is build, or an
	 * existing model is used for evaluation.
	 * @author Michiel Van Bel
	 */
	public enum COMMAND{/**Build a new classification model*/BUILD_MODEL,
						/**Load an existing classificationmodel */LOAD_MODEL};
						
	/**
	 * Possible filetypes which can be used when evaluating sequence files.
	 * @author Michiel Van Bel
	 */					
	public enum FILE_TYPE{	/**Single fasta file format: the first line starts 
	 						   with a >-sign, the others are sequences.*/FASTA,
							/**EMBL file format : DO NOT USE--> BUG*/EMBL};
							
	/**
	 * Possible strands which should be evaluated by the classification models.
	 * @author Michiel Van Bel
	 */						
	public enum STRAND{	/**Forward strand only. Forward strand is assumed to be the 
	 						original one.*/FORWARD,
	 					/**Reverse strand only.*/REVERSE,
	 					/**Both strands*/BOTH};
	
	/**
	 * Choice of different available classifiers.
	 * @author Michiel Van Bel
	 */ 					
	public enum CLASSIFIERS{/**java implementation of LibSVM Support vector machine
	 						DO NOT USE :: UNSTABLE*/LIBSVM,
							/**SMO support vector machine implementation in the weka library.*/WEKASVM,
							/**Random forest implementation in the weka library*/RANDOM_FOREST};
	
	//parent workflow, necessary in case of multithreading.
	private FunSiPWorkflow parentWorkflow				= null;
	
	private List<ClassificationFeature> cfList 		= new ArrayList<ClassificationFeature>();
	private List<Conversion> conversionList			= new ArrayList<Conversion>();
	private Logger logger							= null;
	private String name 							= null;
	private String classification_name				= null;
	private COMMAND command							= COMMAND.BUILD_MODEL;
	private CLASSIFIERS classifier_type				= CLASSIFIERS.LIBSVM;		
	
	private Double output_fixed_alpha				= null;
	private Double output_fixed_beta				= null;
	
	private String positive_training_file			= null;
	private String negative_training_file			= null;
	private String positive_training_file_secstruct	= null;
	private String negative_training_file_secstruct = null;
	private String model_file						= null;
	private String features_output_file				= null;
	private int positive_training_amount			= -1;
	private int negative_training_amount			= -1;
	
	private String[] testfiles						= null;
	private String[] predictionOutputFiles			= null;
	private FILE_TYPE testfiles_filetype			= FILE_TYPE.FASTA;
	
	private STRAND strand							= STRAND.BOTH;
	
	private int training_functional_site_position		= -1;
	private String outputdirectory					= "./";
	
	private int crossvalidation_fold				= -1;
	private boolean complexityCrossvalidation		= false;
	private int complexityCrossvalidationNum		= 13;
	private int maximum_crossvalidation				= -1;
	private File crossValidationTabFile				= null;

	private String pattern							= null;
	private int pattern_classification_location		= 0;
	private int pattern_classification_output_f 	= 0;
	private int pattern_classification_output_r		= 0;
	
	private FeatureSelectionStub featureSelection	= null; 
	private String[] classifier_options				= null;
	private FileParser fileParser					= null;
	
	private String[][] optimalisation_strings 		= null;

	
	/**
	 * Constructor for this classification action.
	 * @param logger The logger-object is kept the same throughout the entire workflow,
	 * so all logging-activity is done in the same way.
	 * @param parent The workflow object that contains this classification action. Each
	 * classification action is part of a workflow, and each workflow contains at least
	 * one classification action.
	 */
	public ClassificationAction(Logger logger, FunSiPWorkflow parent){
		this.logger = logger;
		this.parentWorkflow = parent;
		featureSelection = new FeatureSelectionStub(this,logger);
	}

	
	
	/**
	 * This method makes a copy of the current classification action. 
	 * This is necessary when multiple copies of this classification action
	 * are necessary, for example during optimizations.
	 * @return A copy of the current classification action.
	 */
	public ClassificationAction make_copy(){
		ClassificationAction ac = null;
		try{
			ac = (ClassificationAction)this.clone();
		}
		catch(Exception exc){
			logger.error("Exception occured in class "+this.getClass()+" in method make_copy()");
			logger.error(exc);
		}
		return ac;
	}
	
	public Object clone() throws CloneNotSupportedException{
		ClassificationAction action = new ClassificationAction(this.logger,this.parentWorkflow);
		action.setClassification_name(this.classification_name);
		action.setClassificationFeatures(this.getClassificationFeatures());
		action.setClassifier_options(this.getClassifier_options());
		action.setClassifier_type(this.classifier_type);
		action.setCommand(this.command);
		action.setComplexityCrossvalidation(this.complexityCrossvalidation);
		action.setConversionList(this.conversionList);
		action.setCrossvalidation_fold(this.crossvalidation_fold);
		action.setCrossValidationTabFile(this.crossValidationTabFile);
		action.setFeatures_output_file(this.features_output_file);
		action.setFeatureSelection((FeatureSelectionStub)this.featureSelection.clone());
		action.setMaximum_crossvalidation(this.maximum_crossvalidation);
		action.setModel_file(this.model_file);
		action.setName(this.name);
		action.setNegative_training_amount(this.negative_training_amount);
		action.setNegative_training_file(this.negative_training_file);
		action.setNegative_training_file_secstruct(this.negative_training_file_secstruct);
		action.setOutput_fixed_alpha(this.output_fixed_alpha);
		action.setOutput_fixed_beta(this.output_fixed_beta);
		action.setOutputdirectory(this.outputdirectory);
		action.setPattern(this.pattern);
		action.setPattern_classification_location(this.pattern_classification_location);
		action.setPattern_classification_output_f(this.pattern_classification_output_f);
		action.setPattern_classification_output_r(this.pattern_classification_output_r);
		action.setPositive_training_amount(this.positive_training_amount);
		action.setPositive_training_file(this.positive_training_file);
		action.setPositive_training_file_secstruct(this.positive_training_file_secstruct);
		action.setPredictionOutputFiles(this.predictionOutputFiles);
		action.setStrand(this.strand);
		action.setTestfiles(this.testfiles);
		action.setTestfiles_filetype(this.testfiles_filetype);
		action.setTraining_functional_site_position(this.training_functional_site_position);
		return action;
	}
	
	
	/**
	 * Starts the dataflow in a asymptotic thread way. 
	 * Usefull when - for example - the entire workflow is running
	 * in a multi-threaded way.
	 * 
	 * The parent workflow is forced to wait until all classification
	 * actions defined within its domain, are finished.
	 */
	public void run(){
		try{
			start_classification();
		}
		catch(Exception exc){
			logger.fatal("An exception occured during the processing of this classification action : "+this.getName());
			logger.fatal("This is the top-level error catching mechanism of a classification action object.\n" +
				"Dumping exception stacktrace to output. Please contact the developpers with these errors.");
			exc.printStackTrace(System.err);
		}
		try{
			if(this.parentWorkflow!=null)
				this.parentWorkflow.waitForSync();
		}
		catch(InterruptedException exc){
			logger.error("Could not synchronize with parent workflow object.");
			logger.error("Exception occured in class "+this.getClass()+" in method make_copy()");
			logger.error(exc);
		}
	}
	
	
	
	/**
	 * This method starts the actual dataflow within the classification action.
	 * Depending on the various possible parameters and their values, 
	 * different types of action will be taken. 
	 * 
	 * For example, when the classification action is set to "build model"-modus, 
	 * the normal course of action is to extract training data, apply feature 
	 * selection and then build the classification SVM model.
	 */
	public void start_classification(){
		
		//first of all, check if the program is running as a jar-file.
		//if so, change outputdirectory to user-directory.
		boolean couldMakeDirs = true;
		if(FunSiPJar.RUN_FROM_JAR&&(outputdirectory.startsWith(".")||outputdirectory.startsWith("/"))){
			this.outputdirectory = FunSiPJar.USER_DIR+outputdirectory;
			File f = new File(outputdirectory);
			outputdirectory = f.getAbsolutePath();
			if(!(outputdirectory.endsWith("/")||outputdirectory.endsWith("\\")))
				outputdirectory+="/";
			f = new File(outputdirectory);
			if(!f.exists())
				couldMakeDirs = f.mkdirs();
		}
		else{
			File f = new File(outputdirectory);
			outputdirectory = f.getAbsolutePath();
			if(!(outputdirectory.endsWith("/")||outputdirectory.endsWith("\\")))
				outputdirectory+="/";
			f = new File(outputdirectory);
			if(!f.exists())
				couldMakeDirs = f.mkdirs();
		}
		if(!couldMakeDirs){	//creation of necessary output directories failed.
			logger.fatal("Unable to create output directories.");
			return;
		}
		
	
		//initialize the parser that will be used to extract all sequences.
		fileParser = new FileParser();
		logger.info("Starting workflow for action : "+name);
		Classifier classifier = null;
		
		//Compute the various possible combinations of characters of length k.
		//This prevents the recomputation with every call of the classificationfeature(s).
		//Doing it this way instead of the naive intuitive way provides a speed boost of
		//almost 20x, which is very cool.
		computeOptimalisationStrings();
		
		//initial command to prevent data of overwriting each other
		File file = new File(this.outputdirectory+this.features_output_file);
		if(file.exists())
			file.delete();
		
		//switch to the correct type of workflow
		switch(command){
		case BUILD_MODEL : classifier = build_model();break;
		case LOAD_MODEL : classifier = load_model();break;
		}
		if(classifier==null){
			logger.fatal("Classifier reference is null - something went wrong when building or loading the model");
			return;
		}
		if(classifier!=null && crossvalidation_fold<2) //crossvalidation is done during building-phase
			evaluate_model(classifier);
		
		//delete all the files that should not be kept
		deleteTrainingFile();
		logger.info("Ending workflow for "+this.name);
	}
	
	
	/**
	 * Method which creates a classification model for the current type
	 * of supplied training data. 
	 * This process is performed in several stages :
	 * a) Features are extracted from all the training data
	 * b) Features are written to a temporary file
	 * c) If necessary, a feature selection algorithm is applied to the features
	 * d) If crossvalidation is selected, then the features are used for this
	 * crossvalidation.
	 * e) If crossvalidation is not selected, supply the features to the classifier
	 * in order to have it make the classification model.
	 * 
	 * @return The classifier containing the classification model.
	 */
	public Classifier build_model(){
		Classifier classifier = chooseClassifier(classifier_type);
		if(classifier == null){
			logger.fatal("chosen classifier is null??!");
			return null;
		}
		
		/*
		 * Extract the features from the trainingdata and write them
		 * to a temporary file.
		 * If the extraction or storage fails, a null-reference classifier
		 * is returned, resulting in the abortion of this program.
		 */
		try{
			long t1 = System.currentTimeMillis();
			extractFeaturesFromTrainingData(classifier);
			long t2 = System.currentTimeMillis();
			logger.debug("Time needed to extract training features : "+(t2-t1)/1000+" seconds");
			logger.debug("Training features for "+name+" extracted");
		}
		catch(Exception exc){
			logger.fatal("Error while processing training data, unable to continue");
			logger.fatal("Exception occured in class "+this.getClass()+" in method build_model()");
			logger.fatal(exc);
			return null;
		}
		
		/*
		 * Feature selection techniques.
		 * Choose the appropiate type of feature selection technique
		 * and apply it to the extracted features.
		 */
		if(featureSelection.doFeatureSelection()){
			//just making sure pointer is still to same object (//ref. action multiplic. during multithreading).
			featureSelection.setLocalClassificationAction(this); 		
			long time1_fs = System.currentTimeMillis();
			featureSelection.applyFeatureSelectionToTraining(classifier,false);
			long time2_fs = System.currentTimeMillis();
			logger.debug("Time used for performing feature selection : "+(time2_fs-time1_fs)/1000+" seconds");
		}
		
		
		/*
		 * Perform crossvalidation if the crossvalidation fold is set to a number 
		 * greater then 1.
		 * Crossvalidation is independent from the building of a model,
		 * the only things needed are the extracted features.
		 */
		if(crossvalidation_fold>1 && !complexityCrossvalidation){
			logger.info("Performing crossvalidation");
			File file = new File(this.getOutputdirectory()+this.getFeatures_output_file());
			
			CrossValidationOutput output = new CrossValidationOutput(this,logger);
			List<RocCurveData> rocCurveDataNames = new ArrayList<RocCurveData>();
			perform_crossvalidation(classifier,file,rocCurveDataNames,output);
			output.writeResults(new File(this.getOutputdirectory()+classification_name+"_crossvalidation_tab.txt"),rocCurveDataNames);
		}
		
		/*
		 * Complexity crossvalidation is a procedure that is similar to normal
		 * crossvalidation. It constitutes an extra variable: the complexity of the SVM. 
		 */
		else if(crossvalidation_fold>1 && complexityCrossvalidation){
			logger.info("Performing complexity crossvalidation");
			File file = new File(this.getOutputdirectory()+this.getFeatures_output_file());
			performOptimalComplexityCrossvalidation(classifier,file);
		}
		
		/*
		 * Building the classification model with the extracted features.
		 * The classifier-object already contains the links to the file(s) that
		 * contain the extracted features. Therefore it can simply call
		 * the classifier.buildClassifier() - method.
		 */
		else{
			logger.debug("Building classifier for "+name);
			long t1 = System.currentTimeMillis();
			boolean test = classifier.buildClassifier();
			long t2 = System.currentTimeMillis();
			if(test){
				logger.info("classifier build for "+name+" complete");
				logger.debug("Time needed to build classifier : "+(t2-t1)/1000+" seconds");
				if(output_fixed_alpha!=null)
					classifier.setSigmoid_A(output_fixed_alpha);
				if(output_fixed_beta!=null)
					classifier.setSigmoid_B(output_fixed_beta);
			}
			else{
				logger.fatal("classifier build for "+name+" failed");
				logger.fatal("Unable to continue workflow");
				return null;
			}
		}
		return classifier;
	}
	
	
	/**
	 * This method is a more or less a necessary evil. It precomputes
	 * arrays that contain all the possible strings (of an alphabet) 
	 * of a certain length. The correct array is then passed on onto the
	 * method that extracts the features (if necessary). 
	 * By using this approach, we prevent the recomputation of the same
	 * array by each invocation of the extract-features method. Using this
	 * hack literally cuts feature extraction speed to 1/(#of instances).
	 * This may thus vary to about 1/100 to 1/10000.
	 */
	public void computeOptimalisationStrings(){
		if(cfList==null)
			return;
		optimalisation_strings = new String[cfList.size()][];
		char[] DNA_alphabet = {'A','C','G','T'};
		for(int i=0;i<cfList.size();i++){
			ClassificationFeature feature = cfList.get(i);
			Conversion conv = feature.getConversion();
			String [] types = null;
			if(feature.needCompositionalTypes()){
				if(conv==null){	//no conversion, use standard A,C,G,T dinucleotides
					types = ClassificationFeatureUtils.generateGeneralCompositionalTypes(feature.getLength(), DNA_alphabet);
				}
				else{	//use the conversion alphabet
					char[] alphabet = conv.getDifferentCharacters();
					types = ClassificationFeatureUtils.generateGeneralCompositionalTypes(feature.getLength(), alphabet);
				}
			}
			optimalisation_strings[i] = types;
		}
	}
	
	
	/**
	 * This method performs the actual crossvalidation. The method relies on capabilities
	 * provided by the classification-object to do the actual calculations. 
	 * 
	 * @param classifier The classifier-object used for performing the calculations
	 * @param featureFile The file containing the extracted features
	 * @param rocCurveDataNames An empty list to be filled with names of roc-curves, derived
	 * from the crossvalidation results.
	 * @param output Object to which the output of the crossvalidation will be written.
	 */
	public void perform_crossvalidation(Classifier classifier,File featureFile,List<RocCurveData> rocCurveDataNames,CrossValidationOutput output){
		int [] pos_neg = null;
		int trainPos = 0;
		int trainNeg = 0;
		int testPos = 0;
		int testNeg = 0;
		
		IOUtils fileUtils = new IOUtils(logger,this);
		rocCurveDataNames.clear();
		
		/*
		 * Some extra hooks in the crossvalidation method. Necessary for 
		 * the maximum_crossvalidation option.  
		 */
		pos_neg = classifier.getPosNegExamplesInFile(featureFile);
		testPos = pos_neg[0]/crossvalidation_fold;
		testNeg = pos_neg[1]/crossvalidation_fold;
		trainPos = testPos*(crossvalidation_fold-1);
		if(maximum_crossvalidation==-1)
			trainNeg = testNeg*(crossvalidation_fold-1);
		else
			trainNeg = trainPos*maximum_crossvalidation;
		
		/*
		 * Perform the crossvalidation and write the results to different files.
		 */
		CrossValidationResult resCross = classifier.crossValidate(crossvalidation_fold,trainPos,trainNeg);
		fileUtils.writeCrossValidation(resCross, featureFile, trainPos, testPos, trainNeg, testNeg);
		output.addResult(resCross, trainPos, trainNeg, testPos, testNeg);
		
		/*
		 * Write the gnuplot-commands for the roc-curve to a file. 
		 */
		//fileUtils.writeRocGnuPlotCommand(rocCurveDataNames, this.getOutputdirectory()+"roc/");
		logger.debug("crossvalidation for "+name+" done");
	}
	
	
	/**
	 * This method performs the complexity crossvalidation operation. This method is used for
	 * crossvalidating and optimizing the complexity-constant of the support vector machine
	 * at the same time. An exponential row of numbers is used as base for these complexity-
	 * constants. The row is going from 2^-10 'till 2^4 (with 4 being the maximum number 
	 * indicated by the variable set in the configuration file).
	 * 
	 * @param classifier The classifier object.
	 * @param featureFile The file containing the extracte features.
	 */
	public void performOptimalComplexityCrossvalidation(Classifier classifier,File featureFile){
		long poc_time1 = System.currentTimeMillis();
		
		double[] complexities = new double[complexityCrossvalidationNum];
		for(int i=-10;i<(complexityCrossvalidationNum-10);i++){
			complexities[i+10] = Math.pow(2, i);
		}
		
		/*
		 * Do the complexity crossvalidation by iterating the list of candidate
		 * complexity-constants and performing the crossvalidation-operation for 
		 * each of them.
		 */
		CrossValidationOutput output = new CrossValidationOutput(this,logger);
		List<RocCurveData> rocCurveDataNames = new ArrayList<RocCurveData>();
		for(int i=0;i<complexities.length;i++){
			logger.debug(" -->"+crossvalidation_fold+"-crossvalidation for complexity "+complexities[i]);
			SVMOptions options = (SVMOptions)classifier.getOptions();
			options.complexity = complexities[i];
			classifier.setOptions(options);		
			perform_crossvalidation(classifier,featureFile,rocCurveDataNames,output);
		}
		
		/*
		 * write the output of the crossvalidations to a tab-file with the
		 * complexity-constant defined in the first column.
		 */
		output.setExtraInfoLabel("SVM_complex");
		List<String> extraInfo = new ArrayList<String>();
		for(int i=0;i<complexities.length;i++)
			extraInfo.add(""+complexities[i]);
		output.setExtraInfo(extraInfo);
		//create a name that is generally unique
		String cvtbName = this.getOutputdirectory()+classification_name+"_crossvalidation_tab.txt";
		crossValidationTabFile = new File(cvtbName);
		output.writeResults(crossValidationTabFile,rocCurveDataNames);
		
		long poc_time2 = System.currentTimeMillis();
		logger.debug("Time used for complexity_optimalization (total) : "+(poc_time2-poc_time1)/1000+" seconds");
	}
	
	
	/**
	 * This method loads a precomputed classification model into memory.
	 * This step is necessary to allow the classification of various sequences.
	 * @return The classifier-object that is loaded with the classification model.
	 */
	public Classifier load_model(){
		logger.debug("Loading SVM_Model and related files");
		Classifier classifier = chooseClassifier(classifier_type);
		if(classifier == null)
			return null;
		boolean loadingCompleted = classifier.loadClassifier(model_file);
		if(!loadingCompleted)
			return null;
		
		/*
		 * If necessary, the user can define its own sets of sigmoid
		 * variables, to better fit the expectations.
		 */
		if(output_fixed_alpha!=null)
			classifier.setSigmoid_A(output_fixed_alpha);
		if(output_fixed_beta!=null)
			classifier.setSigmoid_B(output_fixed_beta);
		
		/*
		 * If the configuration file indicates the use of a feature selection 
		 * algorithm during the building of the classification model, then the same
		 * created filter is loaded here.
		 */
		if(featureSelection.getFeature_selection_purpose()==FeatureSelectionStub.FEATURE_SELECTION_PURPOSE.LOAD_FILTER){
			List<Integer> filter = featureSelection.loadFSFile(featureSelection.getFeature_selection_input_file());
			featureSelection.setFilter(filter);
		}
		
		logger.debug("SVM_model loaded");
		return classifier;
	}
	
	/**
	 * Evaluate the content of sequence files with the supplied classifier (and
	 * accompagnied classification model). 
	 * Each file is inspected independently, so if a failure occurs during the evaluation
	 * of one of the sequences, then the evaluation can continue for the other ones. 
	 * There might arise some problems though during the merging of the results. 
	 * 
	 * @param classifier The classifier-object containing the classification model.
	 */
	public void evaluate_model(Classifier classifier){
		if(testfiles==null || testfiles.length==0){
			logger.info("No files to be classified.");
			return;
		}
		for(int i=0;i<testfiles.length;i++){
			File file = new File(testfiles[i]);
			if(!file.exists()||file.isDirectory()){
				logger.error("Unable to open the file "+testfiles[i]);
				logger.error("Cannot classify "+testfiles[i]);
				continue;
			}
			long time1 = System.currentTimeMillis();
			try{
				extractAndEvaluateData(file,classifier,i);
			}
			catch(Exception exc){	//so that not all files are forgotten
				logger.error("Exception occured in class "+this.getClass()+" in method make_copy()");
				logger.error("Error occured during processing of file :"+file.getName());
				logger.error(exc);
			}
			long time2 = System.currentTimeMillis();
			logger.debug("Needed time for evaluation of file "+file+" : "+(time2-time1)/1000 +" seconds");
		}	
	}
	
	
	/**
	 * Evaluate the content of a specific file with the supplied classification model.
	 * The choice is made between forward strand only, reverse strand only or to combine
	 * both forward and reverse strand. The results of the classification are hereafter 
	 * written to the indicated output file if defined. 
	 * @param file File containing the sequence to be evaluated.
	 * @param classifier The classifier-object containing the 
	 * @param index The index of the file.
	 * @throws Exception Thrown if the evaluation of a certain file goes wrong.
	 */
	public void extractAndEvaluateData(File file,Classifier classifier, int index)throws Exception{
		logger.debug("Evaluating "+file.getName()+" for pattern "+pattern);
		FastaFileData data = null;
		DNAUtil util = new DNAUtil();
		switch(testfiles_filetype){
		case FASTA : data = fileParser.extractDataFromFastaFile(file);break;
		case EMBL : data = fileParser.extractDataFromFastaEmblFile(file);break;
		}
		String sequence = data.getSequence();

		sequence  = sequence.toUpperCase();	//just in case...
		String reverse_sequence = util.toOppositeStrand(sequence);
		reverse_sequence = reverse_sequence.trim();
		
		/*
		 * choose between forward evaluation, reverse evaluation or both strands evaluation.
		 * Explanation about the use of those dual lists (it looks pretty weird) is given in
		 * the extractAndClassify-method.
		 */
		switch(strand){
		case FORWARD :
			List<Integer> f_sites = util.getPossibleLocations(sequence, pattern,true).getLocation_in_forward();
			String f_file_path = null;
			if(predictionOutputFiles==null || predictionOutputFiles[index]==null)
				f_file_path = this.getOutputdirectory()+file.getName()+"_"+this.getClassification_name()+"_forward.classif";
			else
				f_file_path = this.getOutputdirectory()+predictionOutputFiles[index];
			extractAndClassify(sequence,classifier,f_sites,f_sites,classification_name,f_file_path,pattern_classification_location,pattern_classification_output_f);
			break;
		case REVERSE : 
			DualLocationList dll_r = util.getPossibleLocations(reverse_sequence, pattern,false);
			List<Integer> r_sites_l = dll_r.getLocation_in_local();
			List<Integer> r_sites_o = dll_r.getLocation_in_forward();
			String r_file_path = null;
			if(predictionOutputFiles==null || predictionOutputFiles[index]==null)
				r_file_path= this.getOutputdirectory()+file.getName()+"_"+this.getClassification_name()+"_reverse.classif";
			else
				r_file_path = this.getOutputdirectory()+predictionOutputFiles[index];
			extractAndClassify(reverse_sequence,classifier,r_sites_l,r_sites_o,classification_name+"_rev",r_file_path,pattern_classification_location,pattern_classification_output_r);
			break;
		case BOTH : 
			String b_f_file_path = this.getOutputdirectory()+file.getName()+"_"+this.getClassification_name()+"_forward.classif";
			String b_r_file_path = this.getOutputdirectory()+file.getName()+"_"+this.getClassification_name()+"_reverse.classif";
			DualLocationList dll_b_f = util.getPossibleLocations(sequence, pattern, true);
			DualLocationList dll_b_r = util.getPossibleLocations(reverse_sequence, pattern,false);
			
			List<Integer> b_f_l = dll_b_f.getLocation_in_local();
			List<Integer> b_f_o = dll_b_f.getLocation_in_forward();
			List<Integer> b_r_l = dll_b_r.getLocation_in_local();
			List<Integer> b_r_o = dll_b_r.getLocation_in_forward();
			
			File f_file = extractAndClassify(sequence,classifier,b_f_l,b_f_o,classification_name,b_f_file_path,pattern_classification_location,pattern_classification_output_f);
			File r_file = extractAndClassify(reverse_sequence,classifier,b_r_l,b_r_o,classification_name+"_rev",b_r_file_path,pattern_classification_location,pattern_classification_output_r);
			File [] files = {f_file,r_file};
			File output = null;
			if(predictionOutputFiles==null||predictionOutputFiles[index]==null){
				output = new File(this.getOutputdirectory()+file.getName()+"_"+this.getClassification_name()+".classif");
			//	logger.debug("prediction output files are null or empty");
			}
			else{
				output = new File(this.getOutputdirectory()+predictionOutputFiles[index]);
				logger.debug("outputfile : "+output.getPath());
			//	logger.debug("prediction output files are not null or empty");
			}
			IOUtils.mergeFiles(output, files);
			break;
		}
	}

	
	/**
	 * This method extracts features around functional sites in a sequence, and then uses the
	 * provided classifier to classify that particular functional site.  
	 * @param sequence The sequence in which the functional sites are located.
	 * @param classifier The classifier with associated classification model that is used for evaluation of the functional sites.
	 * @param sites The location of the functional sites in the sequence.
	 * @param output_sites The location of the functional sites when remapped to the forward strand (necessary for output).
	 * @param clas_name The name used for output for the functional site.
	 * @param fileName The name of the file that will contain the results of the evaluation.
	 * @param increase An integer that is used to increase/decrease the position of the functional site relative to the
	 * location of its identifying pattern (e.g. acceptor AG sites: AG is the pattern, but the acceptor splice site is 
	 * located after the G). 
	 * @param increase_output An integer that identifies an additional increase/decrease to the location of the functional site in the
	 * output. This is done in order to accomodate for the difference in counting (informaticians start from 0, biologists from 1),
	 * so the output can be finetuned enough so no post-processing is needed.  
	 * @return The file containing the evaluations of the various functional sites.
	 */
	public File extractAndClassify(String sequence,Classifier classifier,List<Integer>sites,List<Integer> output_sites,String clas_name,String fileName, int increase, int increase_output){
		List<String> res = new ArrayList<String>();
		File file = new File(fileName);
		logger.debug("# of sites to be evaluated for "+clas_name+" : "+sites.size());
		
		/*
		 * A first check to see whether or not the secondary structure must be used.
		 * According to this, secondary structures can be computed on the fly.
		 * 
		 * TODO : Replace all the stuff about the secondary structures and move it to 
		 * the "Conversion"- hooks. This should definitely clear up things, and make it far more 
		 * accesible.
		 */
		boolean sec_struct_extract = mustExtractSecondaryStructures();
		SecondaryStructureData sec_struct_data = null;
		if(sec_struct_extract){
			SecondaryStructureExtraction extraction = new SecondaryStructureExtraction(logger);
			sec_struct_data = extraction.extractStructures(sequence, sites, this.getOutputdirectory(), this.getMaximumSecondaryUpRange(), this.getMaximumSecondaryDownRange());
		}
		
		//get the right conversions for this sequence.
		String[] conversion_strings = getConversionStrings(sequence);
		
		
		/*
		 * Iterate over the list with all the possible functional sites.
		 */
		for(int i=0;i<sites.size();i++){ 
			long time1 = 0;
			long time2 = 0;
			
			int site = sites.get(i);
			int output_site = output_sites.get(i);
			
			List<List<Double>>features = null;
			
			//**************** SPEED DEBUG **********************//
			if(debug_speed)
				time1 = System.nanoTime();
			
			try{
				//no secondary structure features to be extracted
				if(sec_struct_data==null)
					features = extractFeaturesFromSequence(sequence,null,null,conversion_strings,site+increase);
				
				//secondary structure features to be used
				else{
					//all are of the same length and are aligned, so the splicesite as indicated is not usefull
					String seq = sec_struct_data.getSequences().get(i);
					String sec_str = sec_struct_data.getStructures().get(i);
					double energy = sec_struct_data.getEnergies().get(i);
					features = extractFeaturesFromSequence(seq,sec_str,energy,conversion_strings,this.getMaximumSecondaryUpRange()+increase);
				}
			}
			catch(FeatureExtractionException exc){
				logger.error("Exception occured in class "+this.getClass()+" in method extractAndClassify()");
				logger.error(exc);
			}
			
			//**************** SPEED DEBUG **********************//
			if(debug_speed){
				time2 = System.nanoTime();	
				d_s_extractionTime+=(time2-time1);	
				time1 = System.nanoTime();
			}
			
			List<Double> featureList = new ArrayList<Double>();
			for(int j=0;j<features.size();j++){
				List<Double> temp = features.get(j);
				featureList.addAll(temp);
			}

			//if necessary, apply feature selection to the evaluation feature data.
			if(featureSelection.doFeatureSelection()){
				featureList = featureSelection.applyFeatureSelectionToTest(featureSelection.getStandardClassifierFilter(), featureList);
			}
			
			double [] feat = new double[featureList.size()];
			for(int j=0;j<featureList.size();j++)
				feat[j] = featureList.get(j);
			

			Double distance = classifier.classify_single_instance_fast(feat);
			if(distance!=null){
				String result = classifier.to_splice_machine_output(distance, output_site, increase_output, clas_name);
				res.add(result);
			}
			
			//**************** SPEED DEBUG **********************//
			if(debug_speed){
				time2 = System.nanoTime();		
				d_s_evalTime+=(time2-time1);		
			}
		}
		
		//**************** SPEED DEBUG **********************//
		if(sites.size()!=0 && debug_speed){
			logger.debug("SPEED DEBUG : avg extraction time : "+d_s_extractionTime/sites.size());
			logger.debug("SPEED DEBUG : avg evaluation time : "+d_s_evalTime/sites.size());
			d_s_extractionTime = 0;
			d_s_evalTime = 0;
		}
		
		/*
		 * write the results of the evaluation to the correct output file.
		 */
		try{
			BufferedWriter writer = new BufferedWriter(new FileWriter(file));
			for(int i=0;i<res.size();i++){
				writer.write(res.get(i)+"\n");
			}
			writer.flush();
			writer.close();
		}
		catch(Exception exc){
			logger.error("Exception occured in class "+this.getClass()+" in method extractAndClassify()");
			logger.error("Error occured while processing features from testfile");
			logger.error(exc);
			exc.printStackTrace();
		}
		
		return file;
	}
	
	
	/**
	 * This method  extracts the necessary features in order to train a classifier with
	 * these features. It extracts the features for all the sequences in both the positive
	 * and negative trainingfiles. It then merges the resulting 2 files into 1 file,
	 * according to the specifications of the used classifier. 
	 * @param classifier The classifier to be used.
	 */
	public void extractFeaturesFromTrainingData(Classifier classifier) throws Exception{
		List<String> sequences = null;
		SecondaryStructureData ssData = null;
		String fileName = null;
		logger.debug("Extracting positive training data for "+name);
		sequences = fileParser.extractRandomSequencesFromFile(positive_training_file,positive_training_amount);
		if(positive_training_file_secstruct!=null)
			ssData    = fileParser.extractRandomSecondaryStructuresFromFile(positive_training_file_secstruct,sequences.size());
		fileName = "TEMP_DATA_POS_"+classification_name.toUpperCase()+classifier.getFileExtension();
		File positiveTrainingFile = getFeaturesTrainingData(fileName,sequences,ssData,classifier,Classifier.DATA_TYPE.POSITIVE_DATA);
		
		logger.debug("Extracting negative training data for "+name);
		sequences = fileParser.extractRandomSequencesFromFile(negative_training_file,negative_training_amount);
		if(negative_training_file_secstruct!=null)
			ssData	 = fileParser.extractRandomSecondaryStructuresFromFile(negative_training_file_secstruct, sequences.size());
		fileName = "TEMP_DATA_NEG_"+classification_name.toUpperCase()+classifier.getFileExtension();
		File negativeTrainingFile = getFeaturesTrainingData(fileName,sequences,ssData,classifier,Classifier.DATA_TYPE.NEGATIVE_DATA);
		classifier.mergeFeatureFiles(positiveTrainingFile, negativeTrainingFile);
	}
	
	
	/**
	 * This method extracts the features from a trainingfile (positive/negative), and stores the extracted features
	 * into the feature_output_file. This file is then returned to the parent method.
	 * The method's workflow is pretty easy: iterate over all sequences, extract the features for
	 * each sequence and then write the features to a file. 
	 * @param fileName The name of the file that contains the sequences
	 * @param sequences A list with sequences
	 * @param ssData The secondary structure data
	 * @param classifier The selected classifier
	 * @param data_type The datatype (positive,negative,unknown).
	 * @return The file with the features
	 */
	public File getFeaturesTrainingData(String fileName,List<String> sequences, SecondaryStructureData ssData, Classifier classifier, Classifier.DATA_TYPE data_type){
		File file = null;
		List<String> secstruct = null;
		List<Double> energies = null;
		if(ssData!=null){
			secstruct = ssData.getStructures();
			energies = ssData.getEnergies();
		}
		try{			
			for(int i=0;i<sequences.size();i++){		
				String sequence = sequences.get(i);
				String[] conversion_strings = getConversionStrings(sequence);
				String secon = null;
				Double energy = null;
				if(secstruct!=null){
					secon = secstruct.get(i);
					energy = energies.get(i);
				}
				List<List<Double>> list = extractFeaturesFromSequence(sequence,secon,energy,conversion_strings,training_functional_site_position);
				file = classifier.writeTemporaryFeatureData(fileName,list, data_type);
			}			
		}
		catch(Exception exc){
			logger.error("Exception occured in class "+this.getClass()+" in method getFeaturesFromTrainingData()");
			logger.error(exc);
		}
		return file;
	}
	
	/**
	 * This method takes one string as input and produces all the 
	 * conversions of this string
	 * @param sequence The original DNA/RNA sequence
	 * @return The array containing all the conversions.
	 */
	public String[] getConversionStrings(String sequence){
		String[] result = new String[conversionList.size()];
		for(int i=0;i<conversionList.size();i++){
			Conversion c = conversionList.get(i);
			result[i] = c.makeConversion(sequence);
		}
		return result;
	}
	
	/**
	 * This method is part of the main-loop that extracts the different features. 
	 * This particular method extracts all the features for a given (pseudo-)splice site,
	 * by iterating over the list with classificationfeatures, and so by appending the
	 * found features (for each classificationfeature) to the final list. Because this
	 * iteration should always take place in the same order, all features are also placed in 
	 * the same order. 
	 * Of course, care should be taken by the creators of the classificationfeatures to always
	 * produce the same amount of features for a given set of parameters. Otherwise the results
	 * will of course be meaningless.
	 * @param sequence The sequence from which the features should be extracted.
	 * @param secon The secondary structure of the sequence
	 * @param energy  The free energy that is associated with the secondary structure
	 * @param splicesite The location of the splicesite, relative to which the upstreamm/downstream parameters are located
	 * @return A double linked list with features (one list for each classificationfeature)
	 * @throws FeatureExtractionException Thrown when the set parameters are in contradiction with the arguments of the methods.
	 */
	public List<List<Double>> extractFeaturesFromSequence(String sequence,String secon,Double energy,String[]conversion_strings,int splicesite) throws FeatureExtractionException{
		/*
		 * NOTE TO SELF: 
		 * cannot apply conversions here: for trainingdata it would work out all right,
		 * but the testsequences would be translated in their other forms over and over and over
		 * again, thus creating a huge unnecessary bottleneck. 
		 */
		List<List<Double>> list = new ArrayList<List<Double>>();
		for(int i=0;i<cfList.size();i++){
			ClassificationFeature cf = cfList.get(i);
			String energyString = null;
			if(energy!=null)
				energyString = ""+energy;
			String[] seq = {sequence,secon,""+energyString};
			List<Double> res = cf.extractFeatures(seq, splicesite,optimalisation_strings[i],conversionList,conversion_strings);
			list.add(res);
		}
		return list;
	}
	
	
	
	/**
	 * This method attempts to delete the file that contains all 
	 * the features used for training the classifier. Normally this method
	 * should always succeed, but we have noticed that - particulary with LIBSVM - 
	 * there appears to be a lasting file-lock onto the featurefile, which prevents
	 * it from being deleted.
	 * In order to try to minimize this effect (it seems timing-dependent) we stall
	 * the main thread for about 1 second and try to let the java VM release all the 
	 * locks. Thereafter, two different messages are printed according to whether or
	 * not the deletion succeeded. 
	 */
	public void deleteTrainingFile(){
		try{
			File file = new File(this.getOutputdirectory()+this.getFeatures_output_file());
			if(!file.exists())
				return;
			Thread.sleep(1000);
			boolean test = file.delete();
			if(test)
				logger.debug("Attempting to delete training feature file ... OK");
			else
				logger.debug("Attempting to delete training feature file ... FAILED");
		}
		catch(Exception exc){
			logger.error("Exception occured in class "+this.getClass()+" in method deleteTrainingFile()");
			logger.error("Error while deleting training file");
			logger.error(exc);
		}
	}
	
	
	
	/**
	 * This method creates the right classifier implementation
	 * that implements the Classifier interface, according to
	 * the chosen enumeration.
	 * @param cl Indication of classifier to be used
	 * @return Instantiation of the correct classifier.
	 */
	public Classifier chooseClassifier(CLASSIFIERS cl){
		Classifier classifier = null;
		SVMOptions clOptions  = null;
		boolean test = false;
		switch(cl){
		case LIBSVM:
			classifier = new LibSVM_Weka(logger,this);
			clOptions = new SVMOptions();
			clOptions.classifier = CLASSIFIERS.LIBSVM;
			test = clOptions.setParameters(classifier_options);
			if(!test)
				logger.error("Unable to set libsvm_weka svm options");
			else
				classifier.setOptions(clOptions);
			break;
		case WEKASVM:
			classifier = new SMO_Weka(logger,this);
			clOptions = new SVMOptions();
			clOptions.classifier = CLASSIFIERS.WEKASVM;
			test = clOptions.setParameters(classifier_options);
			if(!test)
				logger.error("Unable to set smo_weka svm options");
			else
				classifier.setOptions(clOptions);
			break;
		default : classifier = null; break;
		}
		return classifier;
	}
	
	
	/**
	 * This method determines whether or not the class should extract
	 * the secondary structures, according to the types of classificationfeatures
	 * @return boolean indicating whether or not to extract the secondary structure features
	 */
	public boolean mustExtractSecondaryStructures(){
		boolean result = false;
		for(int i=0;i< cfList.size();i++){
			ClassificationFeature feat = cfList.get(i);
			if(feat instanceof SeconderyStructureClassificationFeature){
				result = true;
				break;
			}
		}
		
		return result;
	}
	
	/**
	 * This method returns the maximum upstreamrange of all ClassificationFeatures that 
	 * implement secondary structure features.
	 * This method is needed to minimize the computational time of inline
	 * RNA folding
	 * @return The maximum upstreamrange of all ClassificationFeatures that 
	 * implement secondary structure features
	 */
	public int getMaximumSecondaryUpRange(){
		/*
		int result = 0;
		for(int i=0;i<cfList.size();i++){
			ClassificationFeature feat = cfList.get(i);
			if(feat instanceof SeconderyStructureClassificationFeature){
				int[] ud = feat.getUpDownRanges();
				if(ud!=null&&ud.length==2&&ud[0]>result)
					result = ud[0];
			}
		}
		return result;
		*/
		return 100;
	}
	
	/**
	 * This method returns the maximum downstreamrange of all ClassificationFeatures that 
	 * implement secondary structure features.
	 * This method is needed to minimize the computational time of inline
	 * RNA folding
	 * @return The maximum downstreamrange of all ClassificationFeatures that
	 * implement seondary structure features
	 */
	public int getMaximumSecondaryDownRange(){
		/*int result = 0;
		for(int i=0;i<cfList.size();i++){
			ClassificationFeature feat = cfList.get(i);
			if(feat instanceof SeconderyStructureClassificationFeature){
				int[] ud = feat.getUpDownRanges();
				if(ud!=null && ud.length==2 && ud[1]>result)
					result = ud[1];
			}
		}
		return result;
		*/
		return 100;
	}
	
	/*
	 * **********************************************************************
	 * **********************************************************************
	 * **********************************************************************
	 * 		Giant bunch of getters and setters, each of them moslty accessed
	 *  	by the ParameterFileParser class, that constructs both the
	 *   	workflow and classificationActions.
	 *   
	 *   	All getters and setters are pretty much self-explanatory, 
	 *   	so we didn't bother writing javadoc for each of them yet. 
	 *   
	 * **********************************************************************
	 * **********************************************************************  	
	 * **********************************************************************
	 */
	
	
	
	public String getPattern() {
		return pattern;
	}
	public void setPattern(String pattern) {
		this.pattern = pattern;
	}
	public int getPattern_classification_location() {
		return pattern_classification_location;
	}
	public void setPattern_classification_location(
			int pattern_classification_location) {
		this.pattern_classification_location = pattern_classification_location;
	}
	public int getNegative_training_amount() {
		return negative_training_amount;
	}
	public void setNegative_training_amount(int negative_training_amount) {
		this.negative_training_amount = negative_training_amount;
	}
	public int getPositive_training_amount() {
		return positive_training_amount;
	}
	public void setPositive_training_amount(int positive_training_amount) {
		this.positive_training_amount = positive_training_amount;
	}
	public STRAND getStrand() {
		return strand;
	}
	public void setStrand(STRAND strand) {
		this.strand = strand;
	}
	public boolean setStrand(String s){
		s = s.toUpperCase();
		for(STRAND str : STRAND.values()){
			if(str.toString().equals(s)){
				setStrand(str);
				return true;
			}
		}
		return false;
	}
	public FILE_TYPE getTestfiles_filetype() {
		return testfiles_filetype;
	}
	public void setTestfiles_filetype(FILE_TYPE testfiles_filetype) {
		this.testfiles_filetype = testfiles_filetype;
	}
	public boolean setTestfiles_filetype(String s){
		s = s.toUpperCase();
		for(FILE_TYPE ft: FILE_TYPE.values()){
			if(ft.toString().equals(s)){
				setTestfiles_filetype(ft);
				return true;
			}
		}
		return false;
	}
	public COMMAND getCommand() {
		return command;
	}
	public void setCommand(COMMAND command) {
		this.command = command;
	}
	public boolean setCommand(String s){
		s = s.toUpperCase();
		for(COMMAND c : COMMAND.values()){
			if(c.toString().equals(s)){
				setCommand(c);
				return true;
			}
		}
		return false;
	}
	public int getCrossvalidation_fold() {
		return crossvalidation_fold;
	}
	public void setCrossvalidation_fold(int crossvalidation_fold) {
		this.crossvalidation_fold = crossvalidation_fold;
	}
	public int getMaximum_crossvalidation() {
		return maximum_crossvalidation;
	}
	public void setMaximum_crossvalidation(int maximum_crossvalidation) {
		this.maximum_crossvalidation = maximum_crossvalidation;
	}
	public String getModel_file() {
		return model_file;
	}
	public void setModel_file(String file) {
		this.model_file = file;
	}
	public String getFeatures_output_file() {
		return features_output_file;
	}
	public void setFeatures_output_file(String features_output_file) {
		this.features_output_file = features_output_file;
	}
	public String[] getTestfiles() {
		return testfiles;
	}
	public void setTestfiles(String[] testfiles) {
		this.testfiles = testfiles;
	}
	public String getNegative_training_file() {
		return negative_training_file;
	}
	public void setNegative_training_file(String negative_training_file) {
		this.negative_training_file = negative_training_file;
	}
	public String getOutputdirectory() {
		return outputdirectory;
	}
	public void setOutputdirectory(String outputdirectory) {
		this.outputdirectory = outputdirectory;
	}
	public String getPositive_training_file() {
		return positive_training_file;
	}
	public void setPositive_training_file(String positive_training_file) {
		this.positive_training_file = positive_training_file;
	}
	public int getTraining_functional_site_position() {
		return training_functional_site_position;
	}
	public void setTraining_functional_site_position(int splicesite) {
		this.training_functional_site_position = splicesite;
	}
	public void setName(String name){
		char[] chars = name.toCharArray();
		for(int i =0; i<chars.length;i++){
			if(chars[i]==' '||chars[i]=='\t'){
				chars[i] = '_';
			}
		}
		this.name = new String(chars);	
	}
	public String getName(){
		return name;
	}
	public void setClassificationFeatures(List<ClassificationFeature> newList){
		this.cfList = newList;
	}
	public void addClassificationFeature(ClassificationFeature cfi){
		cfList.add(cfi);
	}
	public List<ClassificationFeature> getClassificationFeatures(){
		return cfList;
	}
	public String getClassification_name() {
		return classification_name;
	}
	public void setClassification_name(String classification_name) {
		this.classification_name = classification_name;
	}
	public CLASSIFIERS getClassifier_type() {
		return classifier_type;
	}
	public void setClassifier_type(CLASSIFIERS classifier_type) {
		this.classifier_type = classifier_type;
	}
	public boolean setClassifier_type(String s){
		s = s.toUpperCase();
		for(CLASSIFIERS c : CLASSIFIERS.values()){
			if(c.toString().equals(s)){
				setClassifier_type(c);
				return true;
			}
		}
		return false;
	}
	public String getNegative_training_file_secstruct() {
		return negative_training_file_secstruct;
	}
	public void setNegative_training_file_secstruct(
			String negative_training_file_secstruct) {
		this.negative_training_file_secstruct = negative_training_file_secstruct;
	}
	public String getPositive_training_file_secstruct() {
		return positive_training_file_secstruct;
	}
	public void setPositive_training_file_secstruct(
			String positive_training_file_secstruct) {
		this.positive_training_file_secstruct = positive_training_file_secstruct;
	}
	/*public String getClassifier_file_extension() {
		return classifier_file_extension;
	}
	public void setClassifier_file_extension(String classifier_file_extension) {
		this.classifier_file_extension = classifier_file_extension;
	}*/
	public FeatureSelectionStub getFeatureSelection() {
		return featureSelection;
	}
	public void setFeatureSelection(FeatureSelectionStub featureSelection) {
		this.featureSelection = featureSelection;
	}
	public String[] getClassifier_options() {
		return classifier_options;
	}
	public void setClassifier_options(String[] classifier_options) {
		this.classifier_options = classifier_options;
	}
	public String[] getPredictionOutputFiles() {
		return predictionOutputFiles;
	}
	public void setPredictionOutputFiles(String[] predictionOutputFiles) {
		this.predictionOutputFiles = predictionOutputFiles;
	}
	public int getPattern_classification_output_f() {
		return pattern_classification_output_f;
	}
	public void setPattern_classification_output_f(
			int pattern_classification_output_f) {
		this.pattern_classification_output_f = pattern_classification_output_f;
	}
	public int getPattern_classification_output_r() {
		return pattern_classification_output_r;
	}
	public void setPattern_classification_output_r(
			int pattern_classification_output_r) {
		this.pattern_classification_output_r = pattern_classification_output_r;
	}
	public Double getOutput_fixed_alpha() {
		return output_fixed_alpha;
	}
	public void setOutput_fixed_alpha(Double output_fixed_alpha) {
		this.output_fixed_alpha = output_fixed_alpha;
	}
	public Double getOutput_fixed_beta() {
		return output_fixed_beta;
	}
	public void setOutput_fixed_beta(Double output_fixed_beta) {
		this.output_fixed_beta = output_fixed_beta;
	}
	public boolean isComplexityCrossvalidation() {
		return complexityCrossvalidation;
	}
	public void setComplexityCrossvalidation(boolean complexityCrossvalidation) {
		this.complexityCrossvalidation = complexityCrossvalidation;
	}
	public File getCrossValidationTabFile() {
		return crossValidationTabFile;
	}
	public void setCrossValidationTabFile(File crossValidationTabFile) {
		this.crossValidationTabFile = crossValidationTabFile;
	}
	public List<Conversion> getConversionList() {
		return conversionList;
	}
	public void addConversion(Conversion c){
		this.conversionList.add(c);
	}
	public void setConversionList(List<Conversion> conversionList) {
		this.conversionList = conversionList;
	}
}
