/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package base.features;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.Writer;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.apache.batik.dom.GenericDOMImplementation;
import org.apache.batik.svggen.SVGGraphics2D;
import org.apache.log4j.Logger;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;

import util.classificationFeatures.ClassificationFeature;
import util.classificationFeatures.ClassificationFeatureUtils;
import util.io.FileParser;
import util.io.FileUtils;

/**
 * Class designed to take care of selected features by a feature selection
 * algorithm. 
 * 
 * WARNING: class is still definitely in alpha stage, so some functionality is still 
 * missing, known (I have better things to do) and unknown bugs are present, ...
 * 
 * @author Michiel Van Bel
 */
public class FeatureRecognition implements Runnable{

	/**
	 * Size of the blocks of a single feature in an SVG picture. Size is
	 * in pixels.
	 */
	public static final int BLOCKSIZE = 16;
	
	
	/**
	 * What action to do: 
	 * a) WRITE_SVG 	: simply make an SVG picture of the features. Use BATIK for this.
	 * b) FIND_SUPPORT	: After the features have been selected, watch what the support is 
	 * 						for each feature, by examining the source files as well.
	 * c) COMPARE_SCORES: Compare the scores of two different runs of the same workflow
	 * 						and algorithm. Because data is selected in a randomized way,
	 * 						we might have different feature selection filters.
	 * d) EXPL			: Print an explanation string for each retained feature. Usefull
	 * 						for interpretation of the features.
	 * @author Michiel Van Bel
	 */
	public enum TARGET {WRITE_SVG,FIND_SUPPORT,COMPARE_SCORES,EXPL};
	
	/*
	 * Some global variables.
	 */
	private Logger logger					= null;
	private String feature_filename 		= null;
	private String positive_data_filename	= null;
	private String negative_data_filename	= null;
	private String output_dir				= null;
	private int funsite					= 0;
	private int splicesite_increase			= 0;
	private TARGET target = TARGET.FIND_SUPPORT;
	
	
	/**
	 * constructor. Initialization of some basic variables.
	 * @param logger The logging facility (cannot be null).
	 * @param feature_filename The name of the file containing the feature filter.
	 * @param positive_data_filename The name of the file containing positive training data 
	 * 	(can be null in certain cases).
	 * @param negative_data_filename The name of the file containing negative training data
	 * 	(can be null in certain cases).
	 * @param outputDir The directory to which the results should be written.
	 * @param funsite The location of the functional site in the training data.
	 */
	public FeatureRecognition(Logger logger,String feature_filename,String positive_data_filename,
			String negative_data_filename,String outputDir, int funsite){
		if(logger==null)
			throw new NullPointerException();
		this.logger					= logger;
		this.feature_filename 		= feature_filename;
		this.positive_data_filename	= positive_data_filename;
		this.negative_data_filename	= negative_data_filename;
		this.funsite				= funsite;
		this.output_dir 			= outputDir;
	}
	
	/**
	 * Constructor. Initialization of some basic variables.
	 * @param logger The logging facility. 
	 * @param feature_filename The name of the file containing the feature filter.
	 * @param outputDir The location of the directory to which results should be written.
	 * @param splicesite_increase The increase 
	 */
	public FeatureRecognition(Logger logger,String feature_filename,String outputDir,int splicesite_increase){
		if(logger==null)
			throw new NullPointerException();
		this.logger 				= logger;
		this.feature_filename		= feature_filename;
		this.output_dir				= outputDir;
		this.splicesite_increase	= splicesite_increase;
	}
	
	/*
	 * --> I once shot a man in Reno, just to watch him die.
	 * --> I once shot a man in Reno, but I could't tell you why.
	 * --> I once shot a man in Reno, then I went home to cry.
	 * --> I once shot a man in Reno, then I watered his cacti.
	 * --> I once shot a man in Reno, because they cancelled Firefly
	 * --> I once shot a man in Reno, him and all his succubi.
	 * --> I once shot a man in Reno, and a bunch more in My Lai.
	 * --> ... I think we're done.
	 */
	public void run(){
		//get all the possible data from the featurefile
		FeatureFileData ffData = getFeatureFileData(feature_filename);
		if(ffData==null){
			logger.error("Cannot complete FeatureRecognition, cannot parse featurefile");
			return;
		}
		
		logger.debug("Data from the filter-file is extracted");
		
		List<List<Double>> split_scores		= new ArrayList<List<Double>>();
		splitData(ffData,split_scores);	
		logger.debug("The fs-scores are split among their respective feature types");
		
		List<List<Integer>> non_null_score_positions = detectNonNullScorePositions(split_scores);
		
		String outputfile = "./output/features_support.txt";
		
		switch(target){
		case WRITE_SVG: write_svg(ffData,split_scores,non_null_score_positions);break;
		case FIND_SUPPORT : find_support(ffData,split_scores,outputfile,non_null_score_positions);break;
		case COMPARE_SCORES : break;
		case EXPL : write_explanations(ffData,split_scores,outputfile,non_null_score_positions);break;
		}
	}
	
	private void write_explanations(FeatureFileData ffData,List<List<Double>> scores,String fileName,List<List<Integer>> nnPositions){
		try{
			BufferedWriter writer = new BufferedWriter(new FileWriter(new File(fileName)));		
			for(int i=0;i<ffData.cfList.size();i++){	//iterate over all the classificationfeatures
				ClassificationFeature feature = ffData.cfList.get(i);
				System.out.println("Feature stats : "+feature.toStringUnderscore());
				List<Double> feature_scores = scores.get(i);
				List<Integer> positions = nnPositions.get(i);
				for(int j=0;j<positions.size();j++){
					int pos = positions.get(j);
					double score = feature_scores.get(pos);
					StringBuffer final_string_expl = new StringBuffer();
					String feature_explanation = feature.getLocationDescription(pos).toString();
					final_string_expl.append(feature_explanation+" <==> "+score);
					writer.write(final_string_expl.toString()+"\n");
					writer.flush();
				}
			}
			writer.flush();
			writer.close();
		}
		catch(Exception exc){
			logger.error(exc);
			exc.printStackTrace();
		}
	}
	
	
	
	/*
	 * This method tries to display the provided features with SVG graphics.
	 * The resulting svg-files are written to the filesystem, where they can be 
	 * used for either direct viewing or for use by a webserver
	 */
	private void write_svg(FeatureFileData ffData,List<List<Double>>split_scores, List<List<Integer>>non_null_score_positions){
		SVGGraphics2D svgGenerator = createSVGGraphics();
		List<ClassificationFeature> features = ffData.cfList;
		for(int i=0;i<features.size();i++){
			ClassificationFeature feat = features.get(i);
			List<Double> scores = split_scores.get(i);
			try{
				feat.getClassificationFeatureDrawing().draw(svgGenerator, scores, BLOCKSIZE,splicesite_increase);
				String fileName = output_dir+feat.toStringUnderscore()+".svg";
				Writer out = new FileWriter(fileName);
				svgGenerator.stream(out,true);
				out.close();
			}
			catch(Exception exc){
				exc.printStackTrace();
				logger.error(exc);
			}
		}
	}
	
	
	/*
	 * Creation of a SVGGraphics2D object (which is basically an extension of
	 * Graphics2D) that can be used to directly paint to SVG-files.
	 * Just takes care of the necessary DOM/SAX shit
	 */
	private SVGGraphics2D createSVGGraphics(){
		DOMImplementation domImpl =  GenericDOMImplementation.getDOMImplementation();
		String svgNS = "http://www.w3.org/2000/svg";
		Document document = domImpl.createDocument(svgNS, "svg", null);
		SVGGraphics2D svgGenerator = new SVGGraphics2D(document);
		return svgGenerator;
	}
	

	/*
	 * This method tries to find the support for non-zero feature selection scores. 
	 * By examinating the positive and negative data that have led to this score, 
	 * we might determine whether or not the data is actually usefull.
	 */
	private void find_support(FeatureFileData ffData,List<List<Double>> scores,String fileName,List<List<Integer>> nnPositions){
		logger.debug("examining support of selected features among pos/neg trainingdata");
		
		NumberFormat format = NumberFormat.getInstance(Locale.US);
		format.setMaximumFractionDigits(3);
		format.setMinimumFractionDigits(3);
		
		try{
			BufferedWriter writer = new BufferedWriter(new FileWriter(new File(fileName)));
			File posFile = new File(positive_data_filename);
			File negFile = new File(negative_data_filename);
			writer.write("# POSITIVE TRAINING EXAMPLES : "+FileUtils.getLinesInFile(posFile)+"\n");
			writer.write("# NEGATIVE TRAINING EXAMPLES : "+FileUtils.getLinesInFile(negFile)+"\n");
			
			for(int i=0;i<ffData.cfList.size();i++){	//iterate over all the classificationfeatures
				ClassificationFeature feature = ffData.cfList.get(i);
				System.out.println("Feature stats : "+feature.toStringUnderscore());
				List<Double> feature_scores = scores.get(i);
				List<Integer> positions = nnPositions.get(i);
				
				
				double[] positive_support = new double[positions.size()];
				double[] negative_support = new double[positions.size()];
				String[] types = ClassificationFeatureUtils.generateCompositionalTypes(feature.getLength());
				FileParser parser = new FileParser();
				
				List<String> pos_data = parser.extractSequencesFromFile(positive_data_filename);
				for(int j=0;j<pos_data.size();j++){
					String sequence = pos_data.get(j);
					String[] sequences = {sequence};
					List<Double> features = feature.extractFeatures(sequences, funsite, types,null,null); //TODO : apply conversion?
					for(int k=0;k<positions.size();k++){
						int position = positions.get(k);
						double feat = features.get(position);
						positive_support[k]+=feat;
					}
				}
				
				List<String> neg_data = parser.extractSequencesFromFile(negative_data_filename);
				for(int j=0;j<neg_data.size();j++){
					String sequence = neg_data.get(j);
					String[] sequences = {sequence};
					List<Double> features = feature.extractFeatures(sequences, funsite, types,null,null); //TODO : apply conversion?
					for(int k=0;k<positions.size();k++){
						int position = positions.get(k);
						double feat = features.get(position);
						negative_support[k]+=feat;
					}
				}
				
				double pos_data_size = pos_data.size();
				double neg_data_size = neg_data.size();
				for(int j=0;j<positions.size();j++){
					positive_support[j] = positive_support[j]/pos_data_size;
					negative_support[j] = negative_support[j]/neg_data_size;
				}
				
				for(int j=0;j<positions.size();j++){
					int pos = positions.get(j);
					double score = feature_scores.get(pos);
					StringBuffer final_string_expl = new StringBuffer();
					String feature_explanation = feature.getLocationDescription(pos).toString();
					final_string_expl.append(feature_explanation+"\t <==> ");
					final_string_expl.append(format.format(score)+"\t <==> ");
					
					String positive_support_string = format.format(positive_support[j]);
					String negative_support_string = format.format(negative_support[j]);
					final_string_expl.append("positive support : "+positive_support_string+"\t ||");
					final_string_expl.append("negative support : "+negative_support_string+"\t");
					
					if(positive_support[j]>negative_support[j])
						final_string_expl.append("~> POSITIVE_SUPPORT");
					else
						final_string_expl.append("~> NEGATIVE SUPPORT");
					
					writer.write(final_string_expl.toString()+"\n");
					writer.flush();
					
				}
			}
			
			writer.flush();
			writer.close();
		}
		catch(Exception exc){
			logger.error(exc);
			exc.printStackTrace();
		}
	}
	
	
	public void printScores(List<Double> scores){
		for(int i=0;i<scores.size();i++){
			double score = scores.get(i);
			if(score!=0 && score !=0.0)
				System.out.println(i+" --> "+score);
		}
	}
	
	public void printFeatures(List<Double> features){
		for(double d: features){
			System.out.print(d+",");
		}
		System.out.println();
	}
	
	private List<List<Integer>> detectNonNullScorePositions(List<List<Double>> scores){
		List<List<Integer>> result = new ArrayList<List<Integer>>();
		for(int i=0;i<scores.size();i++){
			List<Double> scores_list = scores.get(i);
			List<Integer> positions = new ArrayList<Integer>();
			for(int j=0;j<scores_list.size();j++){
				double score = scores_list.get(j);
				if(score!=0 && score != 0.0)
					positions.add(j);
			}
			result.add(positions);
		}
		
		return result;
	}
	
	private void splitData(FeatureFileData ffData, List<List<Double>> scores){
		//just adding the right amount of lists to all solutions
		int cf_num = ffData.cfList.size();
		for(int i=0;i<cf_num;i++){
			List<Double> score = new ArrayList<Double>();
			scores.add(score);
		}
		
		//iterate over all features, and split the data accordingly
		int total_num_feats = 0;
		int current_num_feats = 0;
		for(int i=0;i<cf_num;i++){
			ClassificationFeature feat = ffData.cfList.get(i);
			int num_feats = feat.getNumberOfAttributes();
			current_num_feats = total_num_feats;
			total_num_feats+=num_feats;
			//assume the positions are sorted
			int j=current_num_feats;
			for(;j<total_num_feats;j++){
				scores.get(i).add(ffData.scores.get(j));	//get the score from the long list, put it in the local list
			}
			
			if(i==cf_num-1){	//last test, to see whether or not all scores are used/defined
				int num_scores = ffData.scores.size();
				if(num_scores!=total_num_feats){ //problem, there are some restants of scores
					logger.fatal("Fatal error during pre-processing of the feature-data.");
					logger.fatal("#features designated by classificationfeatures != #features in score-file");
					logger.fatal("==> "+total_num_feats+" != "+num_scores);
					System.exit(1);
				}
			}
		}
		System.out.println("Total number of features : "+total_num_feats);
		for(int i=0;i<cf_num;i++)
			System.out.println("Feature "+i+" :: "+ffData.cfList.get(i)+" :: "+scores.get(i).size());
	}
	
	
	
	
	private FeatureFileData getFeatureFileData(String filename){
		try{
			FeatureFileData data = new FeatureFileData();
			BufferedReader reader = new BufferedReader(new FileReader(new File(filename)));
			String s = reader.readLine();
			int counter = 0;
			while(s!=null){
				if(s.startsWith("CLASSIFICATION_FEATURE")){
					String[] split = s.split("\t");
					ClassificationFeature feature = ClassificationFeatureUtils.parseFeatureFromString(split[1]);
					data.cfList.add(feature);
				}
				else{
					String[] split = s.split("\t");
					double score = 0;
					
					if(split.length==1){ //only 1 number, this is the score
						score = Double.parseDouble(split[0]);
					}
					else{				//if there are 2 numbers, first one is the position, second one is the score
						score = Double.parseDouble(split[1]);
					}
					if(counter!=0)
						data.scores.add(score);
					counter++;
				}
				s = reader.readLine();
			}
			//data.scores.remove(0);	//class-index --> should be discarded
			reader.close();
			return data;
		}
		catch(Exception exc){
			logger.error(exc);
			return null;
		} 
	}
	
	
	private class FeatureFileData{
		public List<ClassificationFeature> cfList = new ArrayList<ClassificationFeature>();
	//	public List<Integer> positions = new ArrayList<Integer>();
		public List<Double> scores	= new ArrayList<Double>();
		
	}


	public TARGET getTarget() {
		return target;
	}


	public void setTarget(TARGET target) {
		this.target = target;
	}
	
}
