/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package classifier;

import base.ClassificationAction.CLASSIFIERS;

/**
 * Class defining various classifier options that are specific
 * to Support vector machines. Access to variables is public for 
 * easier implementation.
 * @author Michiel Van Bel
 */
public class SVMOptions implements ClassifierOptions{

	
	
	/**
	 *	The different kernel functions. 
	 */
	public enum KERNEL_TYPE {LINEAR,POLYNOMIAL,RBF,SIGMOID};
	
	
	/**
	 * Complexity of the SVM
	 */
	public double complexity = 1.0;
	
	/**
	 * Kernel function type
	 */
	public KERNEL_TYPE kernel_type = KERNEL_TYPE.LINEAR;
	
	/**
	 * Degree, only applicable for polynomial kernel functions
	 */
	public double degree = 1.0;
	
	/**
	 * for polynomial, rbf and sigmoid kernel function
	 */
	public double gamma = 1.0;
	
	/**
	 * for polynomial and sigmoid kernel
	 */
	public double coeff  = 0.0;
	
	/**
	 * The type of classifier. 
	 */
	public CLASSIFIERS classifier = CLASSIFIERS.WEKASVM;  
	
	
	/**
	 * Sets classifier parameters by parsing them from an array of strings.
	 * Options are set in a way specific for LibSVM
	 * @param params An array of strings containing the various parameters and 
	 * its values.
	 * @return Whether or not setting the parameters succeeded.
	 */
	private boolean setLibSVMOptions(String[] params){
		for(int i=0;i<params.length;i++){
			if(params[i].equals("-K")){
				int type = Integer.parseInt(params[++i]);
				if(type==0) kernel_type = KERNEL_TYPE.LINEAR;
				else if(type==1) kernel_type = KERNEL_TYPE.POLYNOMIAL;
				else if(type==2) kernel_type = KERNEL_TYPE.RBF;
				else if(type==3) kernel_type = KERNEL_TYPE.SIGMOID;
				else return false;
			}
			else if(params[i].equals("-D")){
				degree = Double.parseDouble(params[++i]); 
			}
			else if(params[i].equals("-C")){
				complexity = Double.parseDouble(params[++i]);
			}
			else if(params[i].equals("-g")){
				gamma = Double.parseDouble(params[++i]);
			}
			else if(params[i].equals("-r")){
				coeff = Double.parseDouble(params[++i]);
			}
			else{
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Sets classifier parameters by parsing them from an array of strings.
	 * Options are set in a way specific for WekaSVM
	 * @param params An array of strings containing the various parameters and 
	 * its values.
	 * @return Whether or not setting the parameters succeeded.
	 */
	private boolean setWekaSVMOptions(String[] params){
		for(int i=0;i<params.length;i++){
			if(params[i].equals("-C")){
				complexity = Double.parseDouble(params[++i]);
				return true;
			}
			else{
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Sets classifier parameters by parsing them from an array of strings.
	 * @param params An array of strings containing the various parameters and 
	 * its values.
	 * @return Whether or not setting the parameters succeeded.
	 */
	public boolean setParameters(String[] params){
		if(params==null)
			return true;
		try{
			switch(classifier){
			case LIBSVM:
				return setLibSVMOptions(params); 
			case WEKASVM :
				return setWekaSVMOptions(params);
			default : return false; 
			}
		}
		catch(Exception exc){
			return false;
		}
	}
}
