/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package main;


import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;

import org.apache.log4j.Logger;

import base.ClassificationAction;
import base.FunSiPWorkflow;
import base.ParameterFileParser;


/**
 * Simple auxilary class, dedicated to multiplying a given 
 * configurationfile, in order to conduct similar experiments 
 * at the same time, without having to resort to manually editing
 * some configuration-files.
 * @author Michiel Van Bel
 *
 */
public class FunSiPDuplicate {

	private Logger logger 				= null;
	private String configFileName 		= null;
	private String keyword				= null;
	private Integer startRange			= null;
	private Integer stopRange			= null;
	private Integer stepRange			= null;
	private Integer multiplicity		= null;
	
	private enum WF{MULTIPLY_FILE,RANGE_VARIABLE};
	private WF wf = WF.MULTIPLY_FILE;
	
	/*
	 * Main- method, used for launching the application,
	 * by creating a logging-object and then passing the 
	 * commandline arguments to the program.
	 */
	public static void main(String[] args){
		if(args.length==0){
			System.err.println("Usage : java main.FunSiPDuplicate [options]");
			return;
		}
		Logger logger = Logger.getLogger("FunSiPDuplicate");
		new FunSiPDuplicate(logger,args);
	}
	
	
	/**
	 * Constructor for the program. 
	 * @param logger The logging facility
	 * @param configFile The configuration file that needs to be multiplied
	 * @param num The number of multiplications needed.
	 */
	public FunSiPDuplicate(Logger logger,String[] args){
		if(logger==null)
			return;
		this.logger = logger;
		try{
			boolean b = parseArguments(args);
			if(!b){
				logger.error("An error occured while parsing the commanline arguments.");
				return;
			}
			
			switch(wf){
			case MULTIPLY_FILE : doMultiFile();break;
			case RANGE_VARIABLE : doRangeVar();break;
			default: break;
			}
			
			
		}
		catch(Exception exc){
			logger.error(exc);
		}
	}

	
	private void doMultiFile(){
		for(int j=0;j<this.multiplicity;j++){
			int index = j+1;
			ParameterFileParser parser = new ParameterFileParser(logger);
			FunSiPWorkflow flow = parser.parseFile(configFileName);
			String outputDir = null;
			for(int i=0;i<flow.getClassificationActions().size();i++){
				ClassificationAction action = flow.getClassificationActions().get(i);
				//assume that all actions have the same outputdirectory in the same
				//workflow.
				if(outputDir==null){
					outputDir = action.getOutputdirectory();
					break;
				}
			}
			doMultiFile2(flow,index,outputDir);
		}

	}
	
	private void doMultiFile2(FunSiPWorkflow flow,int index,String outputDir){
		try{
			for(int i=0;i<flow.getClassificationActions().size();i++){
				ClassificationAction action = flow.getClassificationActions().get(i);
				action.setOutputdirectory(outputDir.substring(0,outputDir.length()-1)+index+"/");
			}
			String configFileString = ParameterFileParser.getConfigurationFileString(flow);
			String normalFileName = flow.getFileName();
			String newFilePath = outputDir+getNewFile(normalFileName,index);
			BufferedWriter writer = new BufferedWriter(new FileWriter(new File(newFilePath)));
			writer.write(configFileString);
			writer.flush();
			writer.close();
		}
		catch(Exception exc){
			logger.error(exc);
			return;
		}
	}
	
	private String getNewFile(String fileName,int index){
		int d = fileName.lastIndexOf(".");
		if(d==-1)
			d = fileName.length();
		String base = fileName.substring(0,d);
		String temp = base+index+fileName.substring(d,fileName.length());
		return temp;
	}
	
	
	private void doRangeVar(){
		
	}

	public boolean parseArguments(String[] args){
		try{
			for(int i=0;i<args.length;i++){
				String s = args[i];
				if(s.equals("-configfile")){
					this.configFileName = args[++i];
				}
				else if(s.equals("-key")){
					this.keyword = args[++i];
				}
				else if(s.equals("-multi")){
					this.multiplicity = Integer.parseInt(args[++i]);
				}
				else if(s.equals("-range")){
					String[] split = args[++i].split("::");
					this.startRange = Integer.parseInt(split[0]);
					this.stopRange  = Integer.parseInt(split[1]);
					this.stepRange	= Integer.parseInt(split[2]);
				}
			}
			
			/*
			 * Checking the various variables.
			 */
			if(configFileName==null){
				logger.error("No configFileName defined");
				return false;
			}
			if(keyword==null&&multiplicity==null){
				logger.error("Both keyword and multiplicity are null. Define at least " +
						"and at most 1");
				return false;
			}
			if(keyword==null && multiplicity!=null && multiplicity <1 ){
				logger.error("Multiplicity is smaller then 1");
				return false;
			}
			else{
				this.wf = WF.MULTIPLY_FILE;
			}
			if(keyword!=null && multiplicity!=null){
				logger.error("Both a keyword and multiplicity are defined.");
				return false;
			}
			if(keyword!=null &&  multiplicity==null){
				//checking 
				//a ) whteher or not the keyword exists
				//b ) if the ranges are present and valid
				if(startRange==null || stopRange==null || stepRange == null){
					logger.error("Keyword defined, but ranges are invalid or null");
					return false;
				}
				if(startRange<0 || stopRange<startRange || stepRange<0){
					logger.error("Keyword defined, but ranges are invalid or null");
					return false;
				}
				String[] keys = ParameterFileParser.POSSIBLE_KEYS;
				boolean found = false;
				for(int i=0;i<keys.length;i++){
					String key = keys[i];
					if(keyword.equals(key)){
						found = true;
					}
				}
				if(!found){
					logger.error("Keyword defined, but it is invalid");
					return false;
				}
				else{
					this.wf = WF.RANGE_VARIABLE;
				}
			}
		}
		catch(Exception exc){
			logger.error(exc);
		}
		return true;
	}
	
	
}
