/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package main;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.SimpleLayout;

import base.features.FeatureRecognition;


/**
 * The goal of this program - part of the FunSiP-package - is to evaluate,
 * study and display the various features that were selected and scored by
 * the possible feature selection algorithms. 
 * 
 * WARNING : This program is still very much an alpha version, with lots of 
 * functionality not yet implemented and missing documentation.
 * @author Michiel Van Bel
 */
public class FunSiPFeatures {

	/**
	 * Version of the FunSiPFeatures program.
	 */
	public final static String VERSION = "0.0.3";
	
	private Logger logger						= Logger.getLogger(this.getClass());
	
	
	/**
	 * Main method for FunSiP. Launches the program.
	 * @param args The commandline parameters.
	 */
	public static void main(String[] args){
		new FunSiPFeatures(args);
	}
	
	/**
	 * Constructor of the FunSiPFeatures program. Does some commandline
	 * parameter checking and launches the correct subprograms.
	 * @param args The commandline parameters, passed on by the main-method.
	 */
	public FunSiPFeatures(String[] args){
		logger.setLevel((Level)Level.DEBUG);
		ConsoleAppender appender = new ConsoleAppender(new SimpleLayout());
		logger.addAppender(appender);
		logger.info("FunSiPFeatures "+VERSION+" started");
		
		//check whether there are enough arguments, then simply call constructor
		String command = args[0];
		args = removeCommand(args);
		
		if(command.equals("-SVG")){
			svg(args);
		}
		else if(command.equals("-SUPPORT")){
			support(args);
		}
		else if(command.equals("-COMPARE")){
			compare(args);
		}
		else if(command.equals("-EXPL")){
			writeExplanations(args);
		}
		else{
			logger.fatal("Unknown or no command found. Possible options : [-SVG,-SUPPORT,-COMPARE]");
		}
	}
	
	/**
	 * Some basic stupid method to remap the array with commandline
	 * parameters to a new one from which the first parameter is deleted.
	 * @param args The original array 
	 * @return The new array
	 */
	private String[] removeCommand(String[] args){
		String[] result = new String[args.length-1];
		for(int i=1;i<args.length;i++){
			result[i-1]=args[i];
		}
		return result;
	}
	
	
	/**
	 * Writes some explanations about the selected features, in text form.
	 * @param args Array with options: first one is the feature file, second one
	 * is the outputdirectory.
	 */
	private void writeExplanations(String[] args){
		if(args.length!=2){
			logger.fatal("Incomplete number of arguments passed:");
			logger.fatal("Usage: java FunSiPFeatures -EXPL <FS-file> <outputdir>");
			System.exit(1);
		}
		String fsFile = args[0];
		String outputDir = args[1];
		try{
			FeatureRecognition fr = new FeatureRecognition(logger,fsFile,outputDir,0);
			fr.setTarget(FeatureRecognition.TARGET.EXPL);
			Thread t = new Thread(fr);
			t.start();
			t.join();
		}
		catch(Exception exc){
			exc.printStackTrace();
			logger.error(exc);
		}
	}
	
	/**
	 * Creates the SVG-pictures that are linked to the feature files.
	 * @param args Commandline parameters:
	 * first one is the feature file, second one is the outputdir, third one
	 * is the increase.
	 */
	private void svg(String[] args){
		if(args.length!=3){
			logger.fatal("Incomplete number of arguments passed :");
			logger.fatal("Usage: java FunSiPFeatures -SVG <FS-File> <OutputDir> <splicesite_increase>");
			System.exit(1);
		}
		String fsFile = args[0];
		String outputDir = args[1];
		int increase = Integer.parseInt(args[2]);
		try{
			FeatureRecognition fr = new FeatureRecognition(logger,fsFile,outputDir,increase);
			fr.setTarget(FeatureRecognition.TARGET.WRITE_SVG);
			Thread t = new Thread(fr);
			t.start();
			t.join();
		}
		catch(Exception exc){
			exc.printStackTrace();
			logger.error(exc);
		}
	}
	
	/**
	 * Research of the support for a feature file.
	 * @param args
	 */
	private void support(String[]args){
		if(args.length!=5){
			logger.fatal("Incomplete number of arguments passed : ");
			logger.fatal("Usage: java FunSiPFeatures -SUPPORT <FS-File> <PositiveTraining> <NegativeTraining> <Splicesite>");
			System.exit(1);
		}
		
		String fsFile = args[0];
		String posFile = args[1];
		String negFile = args[2];
		String outputDir = args[3];
		int splicesite = Integer.parseInt(args[4]);
			
		try{
			FeatureRecognition fr = new FeatureRecognition(logger,fsFile,posFile,negFile,outputDir,splicesite);
			fr.setTarget(FeatureRecognition.TARGET.FIND_SUPPORT);
			Thread t = new Thread(fr);
			t.start();
			t.join();
		}
		catch(Exception exc){
			logger.error(exc);
		}
	}
	
	/**
	 * Comparing 2 or more feature selection files.
	 * @param args
	 */
	private void compare(String[] args){
		
	}
	
	
}
