/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.classificationFeatures.implementations;

import java.util.ArrayList;
import java.util.List;

import util.classificationFeatures.AttributeDescription;
import util.classificationFeatures.ClassificationFeature;
import util.classificationFeatures.ClassificationFeatureDrawing;
import util.classificationFeatures.ClassificationFeatureUtils;
import util.classificationFeatures.FeatureExtractionException;
import util.classificationFeatures.IncorrectClassificationFeatureException;
import util.conversions.Conversion;


public class BranchpointClassificationFeature implements ClassificationFeature{
	
	private int minimum_upstream = 0;
	private int maximum_upstream = 0;
	private String U2Seq 		 = "AG-UAGUA";
	private int palindrome_length = -1;
	
	public BranchpointClassificationFeature(){
		
	}
	
	/**
	 * This method asks for the up- and downstreamranges of the feature
	 * @return The up/down ranges
	 */
	public int[] getUpDownRanges(){
		int [] res = {maximum_upstream,0};
		return res;
	}
	
	
	/**
	 * This method asks the classificationfeature for its conversion-type.
	 * @return The used conversion type. Null of not appropriate.
	 */
	public Conversion getConversion(){
		return null;
	}
	
	/**
	 * This method sets the new upstream and downstream ranges  
	 * @param up The new upstream range
	 * @param down The new downstream range
	 */
	public void setUpDownRanges(int up, int down){
		//don't do it here, doesn't work
	}
	
	/**
	 * Asks whether the computer needs to precompute compositional types.
	 * When a conversion is taken with a large alphabet and/or a large length,
	 * this computation can take WAAAY to long (and take too much memory as well).
	 * Therefore, prior to this operation, this flag should be checked.
	 * @return Whether or not to supply compositional types of certain length.
	 */
	public boolean needCompositionalTypes(){
		return false;
	}
	
	/**
	 * This method extracts the features from a sequence (relative to the splicesite)
	 * and returns those features in a list of double-values (standardized on double
	 * values because of its generic but still small-sized nature).
	 * @param sequences The sequence from which the features should be extracted.
	 * @param splicesite The location of the splicesite (pseudo or real)
	 * @param types The compositional types (if requested, otherwise null).
	 * @param conversions A list with all the conversion types
	 * @param conversion_sequences  An array with all the converted sequences.
	 * @return A list with the features
	 * @throws FeatureExtractionException Thrown when something goes wrong when extracting the features.
	 */
	public List<Double> extractFeatures(String sequences[], int splicesite, String[] types, List<Conversion> conversions,String[]conversion_sequences) throws FeatureExtractionException{
		if(sequences==null||sequences.length<1||sequences[0]==null)
			throw new FeatureExtractionException("Stringsequence is null or could not be retrieved for branchpoint feature.");
		String sequence = sequences[0];
		if(splicesite<0 || splicesite>sequence.length())
			throw new FeatureExtractionException("Splicesite/ranges is outside permitted range for branchpoint feature");

		if(minimum_upstream>splicesite){
			//impossible to extract any features, return an empty list of the right size
			List<Double> res = new ArrayList<Double>();
			res.add(0.0);
			res.add(0.0);
			res.add(0.0);
			res.add(0.0);
			res.add(0.0);
			return res;
		}
		
		if(maximum_upstream>splicesite){
			//minimum_upstream is smaller then splicesite --> otherwise method would have returned by now
			maximum_upstream = splicesite-1;
		}
		
		

		double bestposition = 0;
		double bestScoreSum = 0;
		double bestScoreMul = 0;
		double interAG = 0;
		double pyrCount = 0;
		
		//redefining the minimum and maximum
		
		String newU2Seq = reverseString(U2Seq);
		int loc_dash = newU2Seq.indexOf('-');
		for(int i=splicesite-maximum_upstream+loc_dash;i<=splicesite-minimum_upstream;i++){
			if(sequence.charAt(i)=='A'){
				double[] scores = getBasePairScore(sequence,i,newU2Seq.toCharArray(),loc_dash);
				double sumScore = scores[0];
				double mulScore = scores[1];
				if(sumScore>bestScoreSum){
					bestScoreSum = sumScore;
					bestScoreMul = mulScore;
					bestposition = i;
					interAG = countInterAg(sequence,i,splicesite-1);
					pyrCount = countPyrimidines(sequence,i,splicesite);
				}
			}
		}
		
		List<Double> result = new ArrayList<Double>();
		double pos = (double)(splicesite-bestposition)/(double)(maximum_upstream);
		
		bestScoreSum = bestScoreSum/((newU2Seq.length()-1)*6);
		bestScoreMul = bestScoreMul/Math.pow(4,(newU2Seq.length()-1));
		interAG 	 = Math.pow(2, -1*interAG);
		pyrCount	 = pyrCount/maximum_upstream;
		
		result.add(pos);
		result.add(bestScoreSum);
		result.add(bestScoreMul);
		result.add(interAG);
		result.add(pyrCount);
		
		//adding the palindrom features
		if(palindrome_length!=-1){
			int temp = 0;
			if(bestposition-minimum_upstream<=0){
			//	System.out.println("start : "+(bestposition-minimum_upstream));
				 temp = 0;
				//add zeroes
			}
			else{
			//	System.out.println("start : "+(bestposition-minimum_upstream));
				temp = findPalindromes((int)bestposition,sequence,splicesite);
			}
			result.add(new Double(temp));
		}
		
		return result;
	}
	
	private int findPalindromes(int bp,String sequence,int splicesite){
		int num = 0;
		int x = bp-minimum_upstream;
		int y= bp+minimum_upstream; 
		if(y>sequence.length())
			y = sequence.length();
		
		for(int i=x;i<bp;i++){
			String seq = sequence.substring(i,i+palindrome_length);
			String seq2 = inverseString(seq);
			double maxScore = getBPScore(seq,seq2);
		//	System.out.println(maxScore);
			for(int j=bp;j<y;j++){
				String seq3 = sequence.substring(j,j+palindrome_length);
				double score = getBPScore(seq2,seq3);
				if(score>=0.7*maxScore)
					num++;
			}
		}
		
		return num;
	}

	private String inverseString(String s){
		char[] array = s.toCharArray();
		char[] res = new char[array.length];
		for(int i=0;i<array.length;i++){
			switch(array[i]){
			case 'A' : case 'a' : res[i]='T'; break;
			case 'C' : case 'c' : res[i]='G'; break;
			case 'G' : case 'g' : res[i]='C'; break;
			case 'T' : case 't' : res[i]='A'; break;
			default : res[i]='N';break;
			}
		}
		String t = new String(res);
		return t;
	}
	
	/**
	 * Finds the number of other possible acceptorsites ("AG") between two locations
	 * (mostly used between a branchpoint and an acceptorsite)
	 * @param sequence The sequence in which the number of AG's should be counted
	 * @param start The starting point in the sequence
	 * @param stop The stop point in the sequence
	 * @return The number of AG's between 2 sites.
	 */
	public int countInterAg(String sequence,int start,int stop){
		int count = 0;
		for(int i=start;i<stop-1;i++){
			if(sequence.substring(i,i+2).equals("AG"))
				count++;
		}
		return count;
	}
	
	/**
	 * Counts the number of pyrimidines between the possible splicesite and the possible acceptorsite
	 * @param sequence The sequence in which the pyrimidines should be counted
	 * @param start startlocation from which to start counting
	 * @param stop Stoplocation from which to stop counting
	 * @return The number of pyrimidines in the sequence between a start and stop point.
	 */
	public int countPyrimidines(String sequence,int start,int stop){
		int result = 0;
		for(int i=start;i<stop;i++){
			char t = sequence.charAt(i);
			if(t=='C'||t=='T')
				result++;
		}
		return result;
	}
	
	/**
	 * A basepairmatrix, which describes the binding force between different nucleotides.
	 * Order is : A-C-G-T/U in both horizontal and vertical order
	 */
	public static final double[][] BASEPAIRMATRIX = 
	{{0,0,3,5},
	{0,1,6,0},
	{3,6,1,4},
	{5,0,4,0}};

	/**
	 * Returns the place wihtin the basepairbinding matrix for
	 * a nucleotide
	 * @param c The nucleotide 
	 * @return The place within matrix BASEPAIRMATRIX
	 */
	public int getBPVal(char c){
		int res = 0;
		switch(c){
		case 'A': case 'a' : res = 0; break;
		case 'C': case 'c' : res = 1; break;
		case 'G': case 'g' : res = 2; break;
		case 'T': case 't' : case 'U' : case 'u' : res = 3; break;
		case 'N': res = -1; break;
		default : System.out.println("wrong char : "+c); res = -1;break;
		}
		return res;
	}
	
	private double getBPScore(String seq1,String seq2){
		if(seq1==null||seq2==null) return 0;
		if(seq1.length()!=seq2.length()) return 0;
		
		char[] array1 = seq1.toCharArray();
		char[] array2 = seq2.toCharArray();
		
		double scoreSum = 0;
		
		for(int i=0;i<array1.length;i++){
			int row = getBPVal(array1[i]);
			int column = getBPVal(array2[i]);
			if(row!=-1&&column!=-1){
				scoreSum+=BASEPAIRMATRIX[row][column];
			}
		}
		return scoreSum;
		
	}
	
	/**
	 * Computes basepairscores for possible branchpoints
	 * @param sequence The sequence
	 * @param location Location of the possible branchpoint
	 * @param u2RNA The U2RNA sequence
	 * @param loc_dash The location of the binding-nucleotide within the u2RNA sequence
	 * @return An array which contains at its first position a summation score,
	 * and at its second position a multiplication score.
	 */
	public double[] getBasePairScore(String sequence,int location,char[] u2RNA, int loc_dash){
		double scoreSum = 0;
		double scoreMul = 1;
		
		for(int i=0;i<loc_dash;i++){
			int row = getBPVal(sequence.charAt(location-loc_dash+i));
			int column = getBPVal(u2RNA[i]);
			if(row!=-1 && column!=-1){
				scoreSum+=BASEPAIRMATRIX[row][column];
				scoreMul*=BASEPAIRMATRIX[row][column];
			}
			else{
				//add 0 to scoresum and multiply scoremul with 1
			}
		}
		for(int i=loc_dash+1;i<u2RNA.length;i++){
			int row = getBPVal(sequence.charAt(location-loc_dash+i));
			int column = getBPVal(u2RNA[i]);
			if(row!=-1 && column !=-1){
				scoreSum+=BASEPAIRMATRIX[row][column];
				scoreMul*=BASEPAIRMATRIX[row][column];
			}
			else{
				//add 0 to scoresum and multiply scoremul with 1
			}
		}
		double[] res = {scoreSum,scoreMul};
		return res;
	}
	
	/**
	 * reverses the U2RNA string 
	 * @param string Original U2RNA sequence
	 * @return Reversed U2RNA sequence
	 */
	public String reverseString(String string){
		char[] res = new char[string.length()];
		int length = string.length();
		for(int i=0;i<length;i++)
			res[length-i-1] = string.charAt(i);
		return new String(res);
	}
	
	
	/**
	 * Returns the string representation of the classificationfeature,
	 * but with all possible spaces and tabs replaced by underscores.
	 * @return Alternative string representation
	 */
	public String toStringUnderscore(){
		return "BRANCHPOINT_"+minimum_upstream+"_"+maximum_upstream;
	}
	
	
	/**
	 * String representation of the classificationFeature
	 * @return String representation of the classificationfeature
	 */
	public String toString(){
		return "BRANCHPOINT "+minimum_upstream+" "+maximum_upstream;
	}
	
	/**
	 * String representation of the classificationFeature that contains
	 * the core elements, and discards the upstream/downstream range
	 * elements.
	 * @return Shortened version of the string representation of the classificationfeature
	 */
	public String toNoRangeString(){
		return "BRANCHPOINT";
	}
	
	/**
	 * Returns the number of attributes that are created by this classificationfeature,
	 * according to the internal parameters (e.g. up- and downstreamranges). When the 
	 * internal parameters are not set, or they are in an incorrect state, this method will
	 * return -1 as value
	 * @return The number of attributes created by this classificationfeature
	 */
	public int getNumberOfAttributes(){
		return 5;
	}
	
	/**
	 * Returns a standardized description of the location that is indicated.
	 * e.g. 5 bp upstream of splicesite. In case the location is outside the range
	 * of the feature, an error string is returned.
	 * @param loc The location
	 * @return The description
	 */
	public AttributeDescription getLocationDescription(int loc){
		AttributeDescription description = new AttributeDescription();
		description.type = ClassificationFeatureUtils.CLASSIFICATIONFEATURE_TYPES.BRANCH_POINT;
		description.findSupport = false;
		if(loc==0){
			description.explanation = "Position upstream of acceptor";
		}
		else if(loc==1)
			description.explanation = "Multiplication score for U2RNA";
		else if(loc==2)
			description.explanation = "Sommation score for U2RNA";
		else if(loc==3)
			description.explanation = "# of AG's between best score and splicesite";
		else if(loc==4)
			description.explanation = "Pyrimidine count";
		else
			return null;
		return description;
			
	}
	
	/**
	 * This method sets the necessary parameters for the classificationfeature. 
	 * The parameters are all generically passed as an array of strings, which of course
	 * need to be decoded by the classificationfeature itself.
	 * @param args The array with the parameters in string format
	 * @throws IncorrectClassificationFeatureException Can be thrown when the parameters
	 * are incorrect (outside possible ranges) and/or the parameters could not be parsed.
	 */
	public void setParameters(String[] args) throws IncorrectClassificationFeatureException{
		if(args.length<3)
			throw new IncorrectClassificationFeatureException("Incorrect number of parameters for branchpoint feature");
		String name = args[0].toUpperCase();
		if(!name.equals("BRANCHPOINT")&&!name.equals("BP"))
			throw new IncorrectClassificationFeatureException("Incorrect naming for branchpoint feature");
		try{
			minimum_upstream = Integer.parseInt(args[1]);
			maximum_upstream = Integer.parseInt(args[2]);
			for(int i=3;i<args.length;i++){
				if(args[i].equals("-U2RNA")){
					U2Seq = args[++i];
				}
				else if(args[i].equals("-pal")){
					palindrome_length = Integer.parseInt(args[++i]);
					if(palindrome_length<=0)
						palindrome_length = -1;
				}
				else{
					throw new IncorrectClassificationFeatureException("Extra parameters for branchpoint feature cannot be parsed correctly");
				}
					
			}
		}
		catch(NumberFormatException exc){
			throw new IncorrectClassificationFeatureException("Integer parameters for branchpoint feature cannot be parsed correctly");
		}
		
		if(minimum_upstream<0 || maximum_upstream<0 || minimum_upstream>=maximum_upstream)
			throw new IncorrectClassificationFeatureException("Indicated ranges for branchpoint features are not permitted");
	}
	
	/**
	 * Because all classificationfeatures need different algorithms to draw them,
	 * according to the specifications, these utilities are also moved to the 
	 * implementations of the ClassificationFeatures. However, as not to clutter them,
	 * we have chosen to place this data in a seperate interface. The programmer thus
	 * has a choice of using one or two classes to implement both interfaces.
	 * 
	 * @return The object which can draw the classification features.
	 */
	public ClassificationFeatureDrawing getClassificationFeatureDrawing(){
		return new BranchpointClassificationFeatureDrawing(this);
	}
	
	public int getMaximum_upstream() {
		return maximum_upstream;
	}

	public int getMinimum_upstream() {
		return minimum_upstream;
	}
	public int getLength(){
		return 0;
	}
	
	/**
	 * Makes a copy of this particular classificationfeature. 
	 * The cloneable interface cannot be used directly because it is only applicable
	 * on classes, not interfaces.
	 * @return A copy of the classificationfeature
	 */
	public ClassificationFeature make_copy(){
		ClassificationFeature res = null;
		try{
			res = (ClassificationFeature)this.clone();
		}
		catch(Exception exc){}
		return res;
	}
	
}
