/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.classificationFeatures.implementations;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.List;

import org.apache.batik.svggen.SVGGraphics2D;

import util.classificationFeatures.ClassificationFeatureDrawing;
import util.classificationFeatures.ClassificationFeatureUtils;
import util.classificationFeatures.IncorrectClassificationFeatureException;

/**
 * This class is designed to draw the scores assigned to 
 * features generated by the positional classificationfeature type.
 * 
 * @author Michiel Van Bel
 *
 */
public class PositionalClassificationFeatureDrawing implements ClassificationFeatureDrawing{

	private PositionalClassificationFeature pcf = null;
	
	public PositionalClassificationFeatureDrawing(PositionalClassificationFeature pcf){
		this.pcf = pcf;
	}
	
	public void draw(Graphics2D g2d, List<Integer> list ) throws IncorrectClassificationFeatureException{
		if(list.size()!=pcf.getNumberOfAttributes()){
			throw new IncorrectClassificationFeatureException();
		}
	}
	
	/**
	 * This method draws the scores assigned to each feature,
	 * that is generated by this corresponding ClassificationFeature. 
	 * 
	 * The normal idea is to let certain FeatureSelection methods assign
	 * a score to each feature, and then draw these scores in a
	 * classificationfeature dependend way.
	 * 
	 * @param svg
	 * @param list
	 * @throws IncorrectClassificationFeatureException
	 */
	public void draw(SVGGraphics2D svg, List<Double> list, int blocksize, int splicesite_increase) throws IncorrectClassificationFeatureException{
		/*
		 *  A difference is made between upstream features and downstream features.
		 *  This can be done by considering the upstream- and downstreamranges from the PositionalClassificationFeature,
		 *  and the respective length.
		 */
		
		//test to see whether the size of the list of scores macthes the expected size 
		if(pcf==null || list.size()!=pcf.getNumberOfAttributes())
			throw new IncorrectClassificationFeatureException();
	
		//get the highest score, needed for better drawing
		double maxvalue = 0;
		for(int i=0;i<list.size();i++){
			double d = list.get(i);
			if(d>maxvalue)
				maxvalue = d;
		}
	
		
	/*	List<List<Double>> up_down_scores = splitScoresUpDown(list);
		List<Double> upScores = up_down_scores.get(0);
		List<Double> downScores = up_down_scores.get(1);*/
		
		int length = pcf.getLength();
		String[] types = ClassificationFeatureUtils.generateCompositionalTypes(length);
		
	/*	List<List<Double>> upScoresSplit = splitScoresToTypes(upScores,types);
		List<List<Double>> downScoresSplit = splitScoresToTypes(downScores,types);*/
		
		List<List<Double>> splitScores = splitScoresToTypes(list,types);
		
		//Some initial drawing
		svg.setFont(new Font(null,Font.BOLD,20));
		svg.drawString(pcf.toString(),20, 20);
		svg.setFont(new Font(null,Font.PLAIN,blocksize/2));
		
		//draw the different labels of the types
		for(int i=0;i<types.length;i++){
			if(i%2==0)
				svg.drawString(types[i],10,50+blocksize*i);
			else
				svg.drawString(types[i],20,50+blocksize*i);
		}
		
		//draw the scores
		for(int i=0;i<splitScores.size();i++){
			List<Double>scores = splitScores.get(i);
			for(int j=0;j<scores.size();j++){
				double score = scores.get(j);
				Color c = createColor(createColorValue(score,maxvalue));
				svg.setColor(c);
				svg.fillRect(40+blocksize*i,30+blocksize*j,blocksize,blocksize);
				svg.setColor(Color.black);
				svg.drawRect(40+blocksize*i,30+blocksize*j,blocksize,blocksize);
			}
		}
		
		int splicesite = pcf.getUpstream_range()+splicesite_increase;
		int rows = types.length;
		
		//draw the splicesite position
		svg.setFont(new Font(null,Font.BOLD,20));
		svg.drawString("|",38+splicesite*blocksize,50+rows*blocksize);
		
		
		
		svg.setFont(new Font(null,Font.ITALIC,15));
		svg.drawString("splicesite", 10+splicesite*blocksize, 85+rows*blocksize);
		
		//draw a color scheme
		for(int i=0;i<250;i++){
			svg.setColor(createColor(i));
			svg.fillRect(40+i*2,100+rows*blocksize,2,30);
		}
		svg.setColor(Color.black);
		svg.setFont(new Font(null,Font.PLAIN,12));
		svg.drawString("0",35,160+rows*blocksize);
		svg.drawString("1",535,160+rows*blocksize);
	}
	
	public Color createColor(double value){
		double val = 250*value;
		int red = (int)(val);
		int green = (int)(250-val);
		double b = (-2.0/125)*val*val + 4*val;
		int blue = (int)b;
		return new Color(red,green,blue);
	}
	
	
	public Color createColor(int value){
		double temp = ((double)value/250);
		return createColor(temp);
	}
	
	public double createColorValue(double value, double maxvalue){
		value = value/maxvalue;
		/*if(value!=0.0){
		value+=0.01;
		value = Math.pow(2,value)/2;
		if(value>1)
			value = 1.0;
		}*/
		return value;
	}
	
	private List<List<Double>> splitScoresToTypes(List<Double>scores,String[] types){
		List<List<Double>> result = new ArrayList<List<Double>>();
		
		int times = scores.size()/types.length;
		for(int i=0;i<times;i++){
			List<Double>temp = new ArrayList<Double>();
			for(int j=0;j<types.length;j++){
				double d = scores.get(i*types.length+j);
				temp.add(d);
			}
			result.add(temp);
		}
		return result;
	}
	
	
	/*
	 * This method splits the list of scores in 2 (upstream, downstream) , depending on the
	 * properties of the PositionalClassificationFeature
	 *
	private List<List<Double>> splitScoresUpDown(List<Double> scores){
		List<List<Double>> result = new ArrayList<List<Double>>();
		int[] up_down = pcf.getUpDownRanges();
		int length = pcf.getLength();
		
		//upstreams feature amount is not cut-off at the end, in contrast to the 
		//downstream feature amount.
		int num_up = (int)(up_down[0]*Math.pow(4,length));
		
		List<Double> upList = new ArrayList<Double>();
		List<Double> downList = new ArrayList<Double>();
		for(int i=0;i<num_up;i++)
			upList.add(scores.get(i));
		for(int i=num_up;i<scores.size();i++)
			downList.add(scores.get(i));
		
		result.add(upList);
		result.add(downList);
		
		return result;
		
	}*/
	
}
