/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.classificationFeatures.implementations;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import util.classificationFeatures.AttributeDescription;
import util.classificationFeatures.ClassificationFeature;
import util.classificationFeatures.ClassificationFeatureDrawing;
import util.classificationFeatures.ClassificationFeatureUtils;
import util.classificationFeatures.FeatureExtractionException;
import util.classificationFeatures.IncorrectClassificationFeatureException;
import util.conversions.Conversion;

public class RFClassificationFeature implements ClassificationFeature {
	
	private int upstream_range = 0;
	private int downstream_range = 0;

	public RFClassificationFeature(){
	}
	
	/**
	 * This method asks for the up- and downstreamranges of the feature
	 * @return The up/down ranges
	 */
	public int[] getUpDownRanges(){
		int [] res = {upstream_range,downstream_range};
		return res;
	}
	
	/**
	 * This method asks the classificationfeature for its conversion-type.
	 * @return The used conversion type. Null of not appropriate.
	 */
	public Conversion getConversion(){
		return null;
	}
	
	
	/**
	 * This method sets the new upstream and downstream ranges  
	 * @param up The new upstream range
	 * @param down The new downstream range
	 */
	public void setUpDownRanges(int up, int down){
		this.upstream_range = up;
		this.downstream_range = down;
	}

	/**
	 * Asks whether the computer needs to precompute compositional types.
	 * When a conversion is taken with a large alphabet and/or a large length,
	 * this computation can take WAAAY to long (and take too much memory as well).
	 * Therefore, prior to this operation, this flag should be checked.
	 * @return Whether or not to supply compositional types of certain length.
	 */ 
	public boolean needCompositionalTypes(){
		return true;
	}
	
	/**
	 * This method extracts the features from a sequence (relative to the splicesite)
	 * and returns those features in a list of double-values (standardized on double
	 * values because of its generic but still small-sized nature).
	 * @param sequences The sequence from which the features should be extracted.
	 * @param splicesite The location of the splicesite (pseudo or real)
	 * @param types The compositional types (if requested, otherwise null).
	 * @param conversions A list with all the conversion types
	 * @param conversion_sequences  An array with all the converted sequences.
	 * @return A list with the features
	 * @throws FeatureExtractionException Thrown when something goes wrong when extracting the features.
	 */
	public List<Double> extractFeatures(String sequences[], int splicesite,String[] types, List<Conversion> conversions,String[]conversion_sequences) throws FeatureExtractionException{
		if(sequences==null||sequences.length<1||sequences[0]==null)
			throw new FeatureExtractionException("Stringsequence is null or could not be retrieved for RF feature");
		String sequence = sequences[0];
		if(splicesite<0 || splicesite>sequence.length())
			throw new FeatureExtractionException("Splicesite is outside permitted range for RF feature");
		
		List<Double> result = new ArrayList<Double>();
		List<Double> temp_list = extractSingleFrameFeatures(sequence,splicesite,0,types);
		result.addAll(temp_list);
		temp_list = extractSingleFrameFeatures(sequence,splicesite,1,types);
		result.addAll(temp_list);
		temp_list = extractSingleFrameFeatures(sequence,splicesite,2,types);
		result.addAll(temp_list);
		return result;
	}
	
	
	private List<Double> extractSingleFrameFeatures(String sequence,int splicesite,int frame,String[] types) throws FeatureExtractionException{
		List<Integer> result = new ArrayList<Integer>();
		int finalUpRange = 0;
		int finalDownRange = 0;
		if(splicesite-upstream_range<0)
			finalUpRange = 0;
		else
			finalUpRange = splicesite-upstream_range;

		if(splicesite+downstream_range>sequence.length())
			finalDownRange = sequence.length()-3; //-3 : to make sure they do not go over the edge
		else
			finalDownRange = splicesite+downstream_range-3; //-3 : to make sure they do not go over the edge
		
		Map<String,Integer> upmap = new HashMap<String,Integer>();
		Map<String,Integer> downmap = new HashMap<String,Integer>();
		//String[] types = ClassificationFeatureUtils.generateCompositionalTypes(3); //always length 3 for codons
		for(int i=0;i<types.length;i++){
			upmap.put(types[i], 0);
			downmap.put(types[i],0);
		}

		//placing the right starting index
		while(Math.abs((splicesite-finalUpRange))%3!=frame)
			finalUpRange++;

		//extracting the actual features
		for(int i=finalUpRange;i<splicesite-3;i+=3){
			String temp = sequence.substring(i,i+3);
			Integer val = upmap.get(temp);
			if(val!=null){
				val++;
				upmap.put(temp,val);
			}
		}
		for(int i=splicesite+frame;i<finalDownRange;i+=3){
			String temp = sequence.substring(i,i+3);
			Integer val = downmap.get(temp);
			if(val!=null){
				val++;
				downmap.put(temp,val);
			}
		}
		
		for(int i=0;i<types.length;i++)
			result.add(upmap.get(types[i]));
		for(int i=0;i<types.length;i++)
			result.add(downmap.get(types[i]));
		
		//placing the results in the right interval
		List<Double> res = new ArrayList<Double>();
		int max= ClassificationFeatureUtils.getMaximum(result);
		if(max!=0){
			for(int i=0;i<result.size();i++){
				double er = (double)result.get(i)/(double)max;
				res.add(er);
			}
		}
		else{
			for(int i=0;i<result.size();i++){
				double er = (double)result.get(i);
				res.add(er);
			}
		}
		return res;
	}
	
	/**
	 * Returns the string representation of the classificationfeature,
	 * but with all possible spaces and tabs replaced by underscores.
	 * @return Alternative string representation
	 */
	public String toStringUnderscore(){
		return "RF_"+upstream_range+"_"+downstream_range;
	}
	
	
	/**
	 * String representation of the classificationFeature
	 * @return String representation of the classificationfeature
	 */
	public String toString(){
		return "RF "+upstream_range+" "+downstream_range;
	}
	
	/**
	 * String representation of the classificationFeature that contains
	 * the core elements, and discards the upstream/downstream range
	 * elements.
	 * @return Shortened version of the string representation of the classificationfeature
	 */
	public String toNoRangeString(){
		return "RF";
	}
	
	
	/**
	 * Returns a standardized description of the location that is indicated.
	 * e.g. 5 bp upstream of splicesite. In case the location is outside the range
	 * of the feature, an error string is returned.
	 * @param loc The location
	 * @return The description
	 */
	public AttributeDescription getLocationDescription(int loc){
		String[] rf_types = ClassificationFeatureUtils.generateCompositionalTypes(3);
		AttributeDescription description = new AttributeDescription();
		description.type = ClassificationFeatureUtils.CLASSIFICATIONFEATURE_TYPES.READING_FRAME;
		
		int rf_pow = (int)Math.pow(4,3);
		int rf_l = loc/rf_pow; int rf_d = loc%rf_pow;
		
		description.value = rf_types[rf_d];
		
		if(rf_l==0){
			description.up_down = "upstream";
			description.reading_frame = 0;
		}
		else if(rf_l==1){
			description.up_down = "downstream";
			description.reading_frame = 0;
		}
		else if(rf_l==2){
			description.up_down = "upstream";
			description.reading_frame =1;
		}
		else if(rf_l==3){
			description.up_down = "downstream";
			description.reading_frame= 1;
		}
		else if(rf_l==4){
			description.up_down = "upstream";
			description.reading_frame = 2;
		}
		else if(rf_l==5) {
			description.up_down = "downstream";
			description.reading_frame = 2;
		}
		else
			return null;
		return description;
	}
	
	/**
	 * Returns the number of attributes that are created by this classificationfeature,
	 * according to the internal parameters (e.g. up- and downstreamranges). When the 
	 * internal parameters are not set, or they are in an incorrect state, this method will
	 * return -1 as value
	 * @return The number of attributes created by this classificationfeature
	 */
	public int getNumberOfAttributes(){
		return 384;
	}
	
	/**
	 * This method sets the necessary parameters for the classificationfeature. 
	 * The parameters are all generically passed as an array of strings, which of course
	 * need to be decoded by the classificationfeature itself.
	 * @param args The array with the parameters in string format
	 * @throws IncorrectClassificationFeatureException Can be thrown when the parameters
	 * are incorrect (outside possible ranges) and/or the parameters could not be parsed.
	 */
	public void setParameters(String[] args) throws IncorrectClassificationFeatureException{
		if(args.length!=3)
			throw new IncorrectClassificationFeatureException("Incorrect number of parameters for RF feature");
		String name = args[0].toUpperCase();
		if(!name.equals("READING_FRAME")&&!name.equals("RF"))
			throw new IncorrectClassificationFeatureException("Incorrect naming for RF feature");
		try{
			upstream_range = Integer.parseInt(args[1]);
			downstream_range = Integer.parseInt(args[2]);
		}
		catch(NumberFormatException exc){
			throw new IncorrectClassificationFeatureException("Integer parameters for RF feature cannot be parsed correctly");
		}
		
		if(upstream_range<0 || downstream_range<0)
			throw new IncorrectClassificationFeatureException("Indicated ranges for RF feature are not permitted");
	}
	
	/**
	 * Because all classificationfeatures need different algorithms to draw them,
	 * according to the specifications, these utilities are also moved to the 
	 * implementations of the ClassificationFeatures. However, as not to clutter them,
	 * we have chosen to place this data in a seperate interface. The programmer thus
	 * has a choice of using one or two classes to implement both interfaces.
	 * 
	 * @return The object which can draw the classification features.
	 */
	public ClassificationFeatureDrawing getClassificationFeatureDrawing(){
		return new RFClassificationFeatureDrawing(this);
	}
	
	
	public int getDownstream_range() {
		return downstream_range;
	}

	public int getUpstream_range() {
		return upstream_range;
	}
	public int getLength(){
		return 3;
	}
	
	/**
	 * Makes a copy of this particular classificationfeature. 
	 * The cloneable interface cannot be used directly because it is only applicable
	 * on classes, not interfaces.
	 * @return A copy of the classificationfeature
	 */
	public ClassificationFeature make_copy(){
		ClassificationFeature res = null;
		try{
			res = (ClassificationFeature)this.clone();
		}
		catch(Exception exc){}
		return res;
	}
}
