/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.classificationFeatures.implementations.test;

import java.util.ArrayList;
import java.util.List;

import util.classificationFeatures.AttributeDescription;
import util.classificationFeatures.ClassificationFeature;
import util.classificationFeatures.ClassificationFeatureDrawing;
import util.classificationFeatures.FeatureExtractionException;
import util.classificationFeatures.IncorrectClassificationFeatureException;
import util.conversions.Conversion;

public class GcContentClassificationFeature implements ClassificationFeature {
	
	private int length = 0;				//framelength
	private int upstream_range = 0;
	private int downstream_range = 0;
	
	public GcContentClassificationFeature(){
	}
	
	/**
	 * This method asks for the up- and downstreamranges of the feature
	 * @return The up/down ranges
	 */
	public int[] getUpDownRanges(){
		int [] res = {upstream_range,downstream_range};
		return res;
	}
	
	/**
	 * This method asks the classificationfeature for its conversion-type.
	 * @return The used conversion type. Null of not appropriate.
	 */
	public Conversion getConversion(){
		return null;
	}
	
	
	/**
	 * This method sets the new upstream and downstream ranges  
	 * @param up The new upstream range
	 * @param down The new downstream range
	 */
	public void setUpDownRanges(int up, int down){
		this.upstream_range = up;
		this.downstream_range = down;
	}

	/**
	 * Asks whether the computer needs to precompute compositional types.
	 * When a conversion is taken with a large alphabet and/or a large length,
	 * this computation can take WAAAY to long (and take too much memory as well).
	 * Therefore, prior to this operation, this flag should be checked.
	 * @return Whether or not to supply compositional types of certain length.
	 */
	public boolean needCompositionalTypes(){
		return false;
	}
	
	/**
	 * This method extracts the features from a sequence (relative to the splicesite)
	 * and returns those features in a list of double-values (standardized on double
	 * values because of its generic but still small-sized nature).
	 * @param sequences The sequence from which the features should be extracted.
	 * @param splicesite The location of the splicesite (pseudo or real)
	 * @param types The compositional types (if requested, otherwise null).
	 * @param conversions A list with all the conversion types
	 * @param conversion_sequences  An array with all the converted sequences.
	 * @return A list with the features
	 * @throws FeatureExtractionException Thrown when something goes wrong when extracting the features.
	 */
	public List<Double> extractFeatures(String sequences[], int splicesite,String[] types, List<Conversion> conversions,String[]conversion_sequences) throws FeatureExtractionException{
		if(sequences==null||sequences.length<1||sequences[0]==null)
			throw new FeatureExtractionException("Stringsequence is null or could not be retrieved.");
		String sequence = sequences[0];
		if(splicesite<0 || splicesite>sequence.length())
			throw new FeatureExtractionException("Splicesite is outside permitted range for positional feature -> splicesite : "+splicesite+" | sequence length : "+sequence.length());
		List<Double> result = new ArrayList<Double>();

		int finalUpRange = 0;
		int finalDownRange = 0;
		boolean finalAddition = false;
		
		//definition of the final ranges
		//with the upstream ranges, extra data is added so all feature vectors have the same length
		if(splicesite-upstream_range<0){
			finalUpRange = 0;
			for(int i=0;i<upstream_range-splicesite;i++){
				result.add(0.0);
			}	
		}
		else
			finalUpRange = splicesite-upstream_range;
		
		//definition of final downstreamrange
		if(splicesite+downstream_range>sequence.length()){
			finalDownRange = sequence.length();
			finalAddition = true;
		}
		else
			finalDownRange = splicesite+downstream_range;
		
		//first, convert sequence to array of same size, defining the gc/gg/cc/cg's
		int[] gcArray = toGcArray(sequence);
		
		//extraction of the features
		for(int i=finalUpRange;i<=finalDownRange-length;i++){
			double count = 0;
			for(int j=0;j<length;j++){
				count+=gcArray[i+j];
			}
			count/=length;
			result.add(count);
		}
		
		//if necessary, add empty types to end 
		if(finalAddition){
			for(int i=0;i<splicesite+downstream_range-finalDownRange;i++){
				result.add(0.0);
			}
		}
		
		return result;
	}
	
	private int[] toGcArray(String seq){
		int[] res = new int[seq.length()];
		res[0] = 0;
		for(int i=1;i<seq.length()-1;i++){
			char t0 = seq.charAt(i-1);
			char t1 = seq.charAt(i);
			char t2 = seq.charAt(i+1);
			
			if(t1=='C'||t1=='G'){
				if(t0=='C'||t0=='G'){
					res[i] = 1;
				}
				else if(t2=='C'||t2=='G'){
					res[i] = 1;
				}
				else{
					res[i] = 0;
				}
			}
			else{
				res[i] = 0;
			}
		}
		
		return res;
	}

	
	/**
	 * Returns the string representation of the classificationfeature,
	 * but with all possible spaces and tabs replaced by underscores.
	 * @return Alternative string representation
	 */
	public String toStringUnderscore(){
		return "GC_"+length+"_"+upstream_range+"_"+downstream_range;
	}
	
	
	/**
	 * String representation of the classificationFeature
	 * @return String representation of the classificationfeature
	 */
	public String toString(){
		return "GC "+length+" "+upstream_range+" "+downstream_range;
	}
	
	/**
	 * String representation of the classificationFeature that contains
	 * the core elements, and discards the upstream/downstream range
	 * elements.
	 * @return Shortened version of the string representation of the classificationfeature
	 */
	public String toNoRangeString(){
		return "GC "+length;
	}
	
	/**
	 * Returns a standardized description of the location that is indicated.
	 * e.g. 5 bp upstream of splicesite. In case the location is outside the range
	 * of the feature, an error string is returned.
	 * @param loc The location
	 * @return The description
	 */
	public AttributeDescription getLocationDescription(int loc){
		return null;
	}
	
	/**
	 * Returns the number of attributes that are created by this classificationfeature,
	 * according to the internal parameters (e.g. up- and downstreamranges). When the 
	 * internal parameters are not set, or they are in an incorrect state, this method will
	 * return -1 as value
	 * @return The number of attributes created by this classificationfeature
	 */
	public int getNumberOfAttributes(){
		int temp = upstream_range+downstream_range;
		return temp;
	}
	
	
	/**
	 * This method sets the necessary parameters for the classificationfeature. 
	 * The parameters are all generically passed as an array of strings, which of course
	 * need to be decoded by the classificationfeature itself.
	 * @param args The array with the parameters in string format
	 * @throws IncorrectClassificationFeatureException Can be thrown when the parameters
	 * are incorrect (outside possible ranges) and/or the parameters could not be parsed.
	 */
	public void setParameters(String[] args) throws IncorrectClassificationFeatureException{
		if(args.length!=4)
			throw new IncorrectClassificationFeatureException("Incorrect number of parameters for gc_content feature");
		String name = args[0].toUpperCase();
		if(!name.equals("GC"))
			throw new IncorrectClassificationFeatureException("Incorrect naming for gc_content feature");
		try{
			length = Integer.parseInt(args[1]);
			upstream_range = Integer.parseInt(args[2]);
			downstream_range = Integer.parseInt(args[3]);
		}
		catch(NumberFormatException exc){
			throw new IncorrectClassificationFeatureException("Integer parameters for gc_content feature cannot be parsed correctly");
		}
		
		if(length<1 || upstream_range<0 || downstream_range<0)
			throw new IncorrectClassificationFeatureException("Indicated ranges for gc_content features are not permitted");
	}
	
	/**
	 * Because all classificationfeatures need different algorithms to draw them,
	 * according to the specifications, these utilities are also moved to the 
	 * implementations of the ClassificationFeatures. However, as not to clutter them,
	 * we have chosen to place this data in a seperate interface. The programmer thus
	 * has a choice of using one or two classes to implement both interfaces.
	 * 
	 * @return The object which can draw the classification features.
	 */
	public ClassificationFeatureDrawing getClassificationFeatureDrawing(){
		return null;
	}

	
	/**
	 * @return The downstream_range for this positional feature
	 */
	public int getDownstream_range() {
		return downstream_range;
	}

	/**
	 * @return The length of the positional feature (most used lengths are 1,2 or 3)
	 */
	public int getLength() {
		return -1;
	}

	/**
	 * @return The upstream_range for this positional feature
	 */
	public int getUpstream_range() {
		return upstream_range;
	}


	public void setDownstream_range(int downstream_range) {
		this.downstream_range = downstream_range;
	}


	public void setLength(int length) {
		this.length = length;
	}


	public void setUpstream_range(int upstream_range) {
		this.upstream_range = upstream_range;
	}
	
	/**
	 * Makes a copy of this particular classificationfeature. 
	 * The cloneable interface cannot be used directly because it is only applicable
	 * on classes, not interfaces.
	 * @return A copy of the classificationfeature
	 */
	public ClassificationFeature make_copy(){
		ClassificationFeature res = null;
		try{
			res = (ClassificationFeature)this.clone();
		}
		catch(Exception exc){}
		return res;
	}
}
