package util.classificationFeatures.implementations.test;

import java.util.ArrayList;
import java.util.List;

import util.classificationFeatures.AttributeDescription;
import util.classificationFeatures.ClassificationFeature;
import util.classificationFeatures.ClassificationFeatureDrawing;
import util.classificationFeatures.ClassificationFeatureUtils;
import util.classificationFeatures.FeatureExtractionException;
import util.classificationFeatures.IncorrectClassificationFeatureException;
import util.conversions.Conversion;

public class IntronRetention1ClassificationFeature implements ClassificationFeature {
	
	private ClassificationFeature cf = null;
	
	public IntronRetention1ClassificationFeature (){
		
	}
	
	/**
	 * This method asks for the up- and downstreamranges of the feature
	 * @return The up/down ranges
	 */
	public int[] getUpDownRanges(){
		int [] res = {-1,-1};
		if(cf!=null)
			return cf.getUpDownRanges();
		else
			return res;
	}
	
	/**
	 * This method asks the classificationfeature for its conversion-type.
	 * @return The used conversion type. Null of not appropriate.
	 */
	public Conversion getConversion(){
		return null;
	}
	
	
	/**
	 * This method sets the new upstream and downstream ranges  
	 * @param up The new upstream range
	 * @param down The new downstream range
	 */
	public void setUpDownRanges(int up, int down){
		//unnecessary
	}

	/**
	 * Asks whether the computer needs to precompute compositional types.
	 * When a conversion is taken with a large alphabet and/or a large length,
	 * this computation can take WAAAY to long (and take too much memory as well).
	 * Therefore, prior to this operation, this flag should be checked.
	 * @return Whether or not to supply compositional types of certain length.
	 */
	public boolean needCompositionalTypes(){
		if(cf!=null)
			return cf.needCompositionalTypes();
		else
			return false;
	}
	
	/**
	 * This method extracts the features from a sequence (relative to the splicesite)
	 * and returns those features in a list of double-values (standardized on double
	 * values because of its generic but still small-sized nature).
	 * @param sequences The sequence from which the features should be extracted.
	 * @param splicesite The location of the splicesite (pseudo or real)
	 * @param types The compositional types (if requested, otherwise null).
	 * @param conversions A list with all the conversion types
	 * @param conversion_sequences  An array with all the converted sequences.
	 * @return A list with the features
	 * @throws FeatureExtractionException Thrown when something goes wrong when extracting the features.
	 */
	public List<Double> extractFeatures(String sequences[], int splicesite,String[] types, List<Conversion> conversions,String[]conversion_sequences) throws FeatureExtractionException{
		if(sequences==null||sequences.length<1||sequences[0]==null)
			throw new FeatureExtractionException("Stringsequence is null or could not be retrieved.");
		String sequence = sequences[0];
		List<Double> result = new ArrayList<Double>();

		
		if(cf!=null){
			result.addAll(cf.extractFeatures(sequences, 20, types, conversions, conversion_sequences));
			result.addAll(cf.extractFeatures(sequences, sequence.length()-20, types, conversions, conversion_sequences));
		}
		return result;
	}
	

	
	/**
	 * Returns the string representation of the classificationfeature,
	 * but with all possible spaces and tabs replaced by underscores.
	 * @return Alternative string representation
	 */
	public String toStringUnderscore(){
		if(cf!=null)
			return "INTRON_RETENTION_1 "+cf.toStringUnderscore();
		else
			return "INTRON RETENTION 1 !INVALID!";
	}
	
	
	/**
	 * String representation of the classificationFeature
	 * @return String representation of the classificationfeature
	 */
	public String toString(){
		if(cf!=null)
			return  "INTRON_RETENTION_1 "+cf.toString();
		else
			return "INTRON RETENTION 1 !INVALID!";
	}
	
	/**
	 * String representation of the classificationFeature that contains
	 * the core elements, and discards the upstream/downstream range
	 * elements.
	 * @return Shortened version of the string representation of the classificationfeature
	 */
	public String toNoRangeString(){
		if(cf!=null)
			return "INTRON_RETENTION_1 "+cf.toNoRangeString();
		else
			return "INTRON RETENTION 1 !INVALID!";
	}
	
	/**
	 * Returns a standardized description of the location that is indicated.
	 * e.g. 5 bp upstream of splicesite. In case the location is outside the range
	 * of the feature, an error string is returned.
	 * @param loc The location
	 * @return The description
	 */
	public AttributeDescription getLocationDescription(int loc){
		int numAtt = getNumberOfAttributes();
		if(cf==null)
			return null;
		if(loc<numAtt/2){
			AttributeDescription ad = cf.getLocationDescription(loc);
			ad.explanation = " exon1/intron1 splicesite ";
			return ad;
		}
		else{
			AttributeDescription ad = cf.getLocationDescription(loc);
			ad.explanation = " intron1/exon2 splicesite ";
			return ad;
		}
	}
	
	/**
	 * Returns the number of attributes that are created by this classificationfeature,
	 * according to the internal parameters (e.g. up- and downstreamranges). When the 
	 * internal parameters are not set, or they are in an incorrect state, this method will
	 * return -1 as value
	 * @return The number of attributes created by this classificationfeature
	 */
	public int getNumberOfAttributes(){
		if(cf!=null)
			return 2*cf.getNumberOfAttributes();
		else
			return 0;
	}
	
	
	/**
	 * This method sets the necessary parameters for the classificationfeature. 
	 * The parameters are all generically passed as an array of strings, which of course
	 * need to be decoded by the classificationfeature itself.
	 * @param args The array with the parameters in string format
	 * @throws IncorrectClassificationFeatureException Can be thrown when the parameters
	 * are incorrect (outside possible ranges) and/or the parameters could not be parsed.
	 */
	public void setParameters(String[] args) throws IncorrectClassificationFeatureException{
		
		String name = args[0].toUpperCase();
		if(!name.equals("INTRON_RETENTION_1")&&!name.equals("IR1"))
			throw new IncorrectClassificationFeatureException("Incorrect naming for intron retention 1 feature");
		
		String[] sub = new String[args.length-1];
		for(int i=1;i<args.length;i++)
			sub[i-1] = args[i];
		ClassificationFeature cf = ClassificationFeatureUtils.parseFeatureFromString(sub);
		if(cf==null)
			throw new IncorrectClassificationFeatureException("Intron retention 1 extra classification feature cannot be parsed");
		cf.setUpDownRanges(20,20);
		this.cf= cf;
	}
	
	/**
	 * Because all classificationfeatures need different algorithms to draw them,
	 * according to the specifications, these utilities are also moved to the 
	 * implementations of the ClassificationFeatures. However, as not to clutter them,
	 * we have chosen to place this data in a seperate interface. The programmer thus
	 * has a choice of using one or two classes to implement both interfaces.
	 * 
	 * @return The object which can draw the classification features.
	 */
	public ClassificationFeatureDrawing getClassificationFeatureDrawing(){
		return null;
	}


	/**
	 * @return The length of the positional feature (most used lengths are 1,2 or 3)
	 */
	public int getLength() {
		if(cf!=null)
			return cf.getLength();
		else
			return 0;
	}
	
	/**
	 * Makes a copy of this particular classificationfeature. 
	 * The cloneable interface cannot be used directly because it is only applicable
	 * on classes, not interfaces.
	 * @return A copy of the classificationfeature
	 */
	public ClassificationFeature make_copy(){
		ClassificationFeature res = null;
		try{
			res = (ClassificationFeature)this.clone();
		}
		catch(Exception exc){}
		return res;
	}
	
}
