package util.classificationFeatures.implementations.test;

import java.util.ArrayList;
import java.util.List;

import util.classificationFeatures.AttributeDescription;
import util.classificationFeatures.ClassificationFeature;
import util.classificationFeatures.ClassificationFeatureDrawing;
import util.classificationFeatures.FeatureExtractionException;
import util.classificationFeatures.IncorrectClassificationFeatureException;
import util.conversions.Conversion;

public class RF2ClassificationFeature implements ClassificationFeature{
	
	
	private int upstream_range 		= 0;
	private int downstream_range 	= 0;

	public RF2ClassificationFeature(){
	}
	
	/**
	 * This method asks for the up- and downstreamranges of the feature
	 * @return The up/down ranges
	 */
	public int[] getUpDownRanges(){
		int [] res = {upstream_range,downstream_range};
		return res;
	}
	
	/**
	 * This method asks the classificationfeature for its conversion-type.
	 * @return The used conversion type. Null of not appropriate.
	 */
	public Conversion getConversion(){
		return null;
	}
	
	
	/**
	 * This method sets the new upstream and downstream ranges  
	 * @param up The new upstream range
	 * @param down The new downstream range
	 */
	public void setUpDownRanges(int up, int down){
		this.upstream_range = up;
		this.downstream_range = down;
	}

	/**
	 * Asks whether the computer needs to precompute compositional types.
	 * When a conversion is taken with a large alphabet and/or a large length,
	 * this computation can take WAAAY to long (and take too much memory as well).
	 * Therefore, prior to this operation, this flag should be checked.
	 * @return Whether or not to supply compositional types of certain length.
	 */
	public boolean needCompositionalTypes(){
		return false;
	}
	
	/**
	 * This method extracts the features from a sequence (relative to the splicesite)
	 * and returns those features in a list of double-values (standardized on double
	 * values because of its generic but still small-sized nature).
	 * @param sequences The sequence from which the features should be extracted.
	 * @param splicesite The location of the splicesite (pseudo or real)
	 * @param types The compositional types (if requested, otherwise null).
	 * @param conversions A list with all the conversion types
	 * @param conversion_sequences  An array with all the converted sequences.
	 * @return A list with the features
	 * @throws FeatureExtractionException Thrown when something goes wrong when extracting the features.
	 */
	public List<Double> extractFeatures(String sequences[], int splicesite,String[] types, List<Conversion> conversions,String[]conversion_sequences) throws FeatureExtractionException{
		if(sequences==null||sequences.length<1||sequences[0]==null)
			throw new FeatureExtractionException("Stringsequence is null or could not be retrieved for RF2 feature");
		String sequence = sequences[0].toUpperCase();
		if(splicesite<0 || splicesite>sequence.length())
			throw new FeatureExtractionException("Functional site is outside permitted range for RF2 feature");
		
		List<Double> result = new ArrayList<Double>();
		char[] alphabet = {'A','C','G','T'};
		
		
		for(int i=0;i<alphabet.length;i++){	//possible letters
			char c = alphabet[i];
			double bestScore = 0;
			for(int j=0;j<3;j++){
				double score = getUpScore(sequence,splicesite,c,j);
				if(score>bestScore)
					bestScore = score;
			}
			result.add(bestScore);
		}
		for(int i=0;i<alphabet.length;i++){	//possible letters
			char c = alphabet[i];
			double bestScore = 0;
			for(int j=0;j<3;j++){
				double score = getDownScore(sequence,splicesite,c,j);
				if(score>bestScore)
					bestScore = score;
			}
			result.add(bestScore);
		}
		
		return result;
	}
	

	public double getUpScore(String seq, int funsite,char c,int shift){
		double result = 0.0;
		int count = 0;
		for(int i=shift;i<upstream_range-3;i=i+3){
			int site = funsite-i; 
			if(site<=0)
				break;
			char loc = seq.charAt(site);
			if(loc==c)
				result++;
			count++;
		}
		if(count>0)
			result = result / count;
		
		return result;
	}
	
	public double getDownScore(String seq, int funsite,char c,int shift){
		double result = 0.0;
		int count = 0;
		for(int i=shift;i<downstream_range-3;i=i+3){
			int site = funsite+i; 
			if(site>seq.length())
				break;
			char loc = seq.charAt(site);
			if(loc==c)
				result++;
			count++;
		}
		if(count>0)
			result = result / count;
		
		return result;
	}
	
	
	/**
	 * Returns the string representation of the classificationfeature,
	 * but with all possible spaces and tabs replaced by underscores.
	 * @return Alternative string representation
	 */
	public String toStringUnderscore(){
		return "RF2_"+upstream_range+"_"+downstream_range;
	}
	
	
	/**
	 * String representation of the classificationFeature
	 * @return String representation of the classificationfeature
	 */
	public String toString(){
		return "RF2 "+upstream_range+" "+downstream_range;
	}
	
	/**
	 * String representation of the classificationFeature that contains
	 * the core elements, and discards the upstream/downstream range
	 * elements.
	 * @return Shortened version of the string representation of the classificationfeature
	 */
	public String toNoRangeString(){
		return "RF2";
	}
	
	
	/**
	 * Returns a standardized description of the location that is indicated.
	 * e.g. 5 bp upstream of splicesite. In case the location is outside the range
	 * of the feature, an error string is returned.
	 * @param loc The location
	 * @return The description
	 */
	public AttributeDescription getLocationDescription(int loc){
		return null;
	}
	
	/**
	 * Returns the number of attributes that are created by this classificationfeature,
	 * according to the internal parameters (e.g. up- and downstreamranges). When the 
	 * internal parameters are not set, or they are in an incorrect state, this method will
	 * return -1 as value
	 * @return The number of attributes created by this classificationfeature
	 */
	public int getNumberOfAttributes(){
		return 8;
	}
	
	/**
	 * This method sets the necessary parameters for the classificationfeature. 
	 * The parameters are all generically passed as an array of strings, which of course
	 * need to be decoded by the classificationfeature itself.
	 * @param args The array with the parameters in string format
	 * @throws IncorrectClassificationFeatureException Can be thrown when the parameters
	 * are incorrect (outside possible ranges) and/or the parameters could not be parsed.
	 */
	public void setParameters(String[] args) throws IncorrectClassificationFeatureException{
		if(args.length!=3)
			throw new IncorrectClassificationFeatureException("Incorrect number of parameters for RF2 feature");
		String name = args[0].toUpperCase();
		if(!name.equals("READING_FRAME2")&&!name.equals("RF2"))
			throw new IncorrectClassificationFeatureException("Incorrect naming for RF2 feature");
		try{
			upstream_range = Integer.parseInt(args[1]);
			downstream_range = Integer.parseInt(args[2]);
		}
		catch(NumberFormatException exc){
			throw new IncorrectClassificationFeatureException("Integer parameters for RF2 feature cannot be parsed correctly");
		}
		
		if(upstream_range<0 || downstream_range<0)
			throw new IncorrectClassificationFeatureException("Indicated ranges for RF2 feature are not permitted");
	}
	
	/**
	 * Because all classificationfeatures need different algorithms to draw them,
	 * according to the specifications, these utilities are also moved to the 
	 * implementations of the ClassificationFeatures. However, as not to clutter them,
	 * we have chosen to place this data in a seperate interface. The programmer thus
	 * has a choice of using one or two classes to implement both interfaces.
	 * 
	 * @return The object which can draw the classification features.
	 */
	public ClassificationFeatureDrawing getClassificationFeatureDrawing(){
		return null;
	}
	
	
	public int getDownstream_range() {
		return downstream_range;
	}

	public int getUpstream_range() {
		return upstream_range;
	}
	public int getLength(){
		return -1;
	}
	
	/**
	 * Makes a copy of this particular classificationfeature. 
	 * The cloneable interface cannot be used directly because it is only applicable
	 * on classes, not interfaces.
	 * @return A copy of the classificationfeature
	 */
	public ClassificationFeature make_copy(){
		ClassificationFeature res = null;
		try{
			res = (ClassificationFeature)this.clone();
		}
		catch(Exception exc){}
		return res;
	}

}
