/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.classificationFeatures.implementations.test;

import java.util.ArrayList;
import java.util.List;

import util.classificationFeatures.AttributeDescription;
import util.classificationFeatures.ClassificationFeature;
import util.classificationFeatures.ClassificationFeatureDrawing;
import util.classificationFeatures.FeatureExtractionException;
import util.classificationFeatures.IncorrectClassificationFeatureException;
import util.conversions.Conversion;

public class SecStructEvalClassificationFeature implements ClassificationFeature{

	
	private int upstream_range = 0;
	private int downstream_range = 0;
	
	public SecStructEvalClassificationFeature(){
		
	}
	
	
	/**
	 * This method asks for the up- and downstreamranges of the feature
	 * @return The up/down ranges
	 */
	public int[] getUpDownRanges(){
		int [] res = {upstream_range,downstream_range};
		return res;
	}
	
	/**
	 * This method asks the classificationfeature for its conversion-type.
	 * @return The used conversion type. Null of not appropriate.
	 */
	public Conversion getConversion(){
		return null;
	}
	
	
	/**
	 * This method sets the new upstream and downstream ranges  
	 * @param up The new upstream range
	 * @param down The new downstream range
	 */
	public void setUpDownRanges(int up, int down){
		this.upstream_range = up;
		this.downstream_range = down;
	}
	
	/**
	 * Asks whether the computer needs to precompute compositional types.
	 * When a conversion is taken with a large alphabet and/or a large length,
	 * this computation can take WAAAY to long (and take too much memory as well).
	 * Therefore, prior to this operation, this flag should be checked.
	 * @return Whether or not to supply compositional types of certain length.
	 */
	public boolean needCompositionalTypes(){
		return false;
	}
	
	/**
	 * This method extracts the features from a sequence (relative to the splicesite)
	 * and returns those features in a list of double-values (standardized on double
	 * values because of its generic but still small-sized nature).
	 * @param sequences The sequence from which the features should be extracted.
	 * @param splicesite The location of the splicesite (pseudo or real)
	 * @param types The compositional types (if requested, otherwise null).
	 * @param conversions A list with all the conversion types
	 * @param conversion_sequences  An array with all the converted sequences.
	 * @return A list with the features
	 * @throws FeatureExtractionException Thrown when something goes wrong when extracting the features.
	 */
	public List<Double> extractFeatures(String sequences[], int splicesite, String[] types, List<Conversion> conversions,String[]conversion_sequences) throws FeatureExtractionException{
		if(sequences==null||sequences.length<3||sequences[0]==null||sequences[1]==null||sequences[2]==null)
			throw new FeatureExtractionException("Stringsequence(s) is/are null or could not be retrieved.");
		String prim_seq 	= sequences[0];
		String sec_struct	= sequences[1];
		if(prim_seq.length()!=sec_struct.length())
			throw new FeatureExtractionException("primary and secondary string lengths are not equal");
		
		List<Double> result = new ArrayList<Double>();
		
		//normally, due to either training data or due to the secondarystructureextraction class
		int start = splicesite-upstream_range;
		int stop  = splicesite+downstream_range;
		
		
		/*
		 * Dit is een snelle implementatie zonder optimalisaties. Na afloop 
		 * moet dit nog eens nader bekeken worden.
		 * features : 
		 * a) # van stems en loops 
		 * 		--> te vinden door tellen van aaneengesloten blokken van ((,)) of .. (minstens 2)
		 * b) gemiddelde lengte van de stems en loops
		 * 
		 */
		
		//first, find all blocs
		List<String> blocList = new ArrayList<String>();
		char current = sec_struct.charAt(start);
		StringBuffer sb = new StringBuffer();
		for(int i=start+1;i<=stop;i++){
			char t = sec_struct.charAt(i);
			if(t==current)
				sb.append(t);
			else{
				current = t;
				if(sb.length()>=2)
					blocList.add(sb.toString());
				sb = new StringBuffer();
			}
		}
		
		
		double open_stem 	= 0;
		double close_stem 	= 0;
		double loop			= 0;
		
		double avg_length_open_stem		= 0;
		double avg_length_close_stem	= 0;
		double avg_length_loop			= 0;
		for(int i=0;i<blocList.size();i++){
			String t = blocList.get(i);
			switch(t.charAt(0)){
			case '(' : 
				open_stem++;
				avg_length_open_stem+=t.length();
				break;
			case ')' :
				close_stem++;
				avg_length_close_stem+=t.length();
				break;
			case '.' :
				loop++;
				avg_length_loop+=t.length();
				break;
			default : break;
			}
		}
		
		avg_length_open_stem/=open_stem;
		avg_length_close_stem/=close_stem;
		avg_length_loop/=loop;
		
		result.add(open_stem);
		result.add(close_stem);
		result.add(loop);
		result.add(avg_length_open_stem);
		result.add(avg_length_close_stem);
		result.add(avg_length_loop);
		
		return result;
	}
	
	/**
	 * Returns the string representation of the classificationfeature,
	 * but with all possible spaces and tabs replaced by underscores.
	 * @return Alternative string representation
	 */
	public String toStringUnderscore(){
		return "SECSTRUCTEVAL_"+upstream_range+"_"+downstream_range;
	}
	
	
	/**
	 * String representation of the classificationFeature
	 * @return String representation of the classificationfeature
	 */
	public String toString(){
		return "SECSTRUCTEVAL "+upstream_range+" "+downstream_range;
	}
	
	/**
	 * String representation of the classificationFeature that contains
	 * the core elements, and discards the upstream/downstream range
	 * elements.
	 * @return Shortened version of the string representation of the classificationfeature
	 */
	public String toNoRangeString(){
		return "SS EVAL ";
	}
	
	/**
	 * Returns the number of attributes that are created by this classificationfeature,
	 * according to the internal parameters (e.g. up- and downstreamranges). When the 
	 * internal parameters are not set, or they are in an incorrect state, this method will
	 * return -1 as value
	 * @return The number of attributes created by this classificationfeature
	 */
	public int getNumberOfAttributes(){
		return 1;
	}
	
	/**
	 * Returns a standardized description of the location that is indicated.
	 * e.g. 5 bp upstream of splicesite. In case the location is outside the range
	 * of the feature, an error string is returned.
	 * @param loc The location
	 * @return The description
	 */
	public AttributeDescription getLocationDescription(int loc){
		return null;
	}
	
	/**
	 * This method sets the necessary parameters for the classificationfeature. 
	 * The parameters are all generically passed as an array of strings, which of course
	 * need to be decoded by the classificationfeature itself.
	 * @param args The array with the parameters in string format
	 * @throws IncorrectClassificationFeatureException Can be thrown when the parameters
	 * are incorrect (outside possible ranges) and/or the parameters could not be parsed.
	 */
	public void setParameters(String[] args) throws IncorrectClassificationFeatureException{
		if(args.length!=3)
			throw new IncorrectClassificationFeatureException("Incorrect number of parameters for ss_eval feature");
		String name = args[0].toUpperCase();
		if(!name.equals("SS_EVAL"))
			throw new IncorrectClassificationFeatureException("Incorrect naming for ss_eval feature");
		try{
			upstream_range	= Integer.parseInt(args[1]);
			downstream_range	= Integer.parseInt(args[2]);
		}
		catch(NumberFormatException exc){
			throw new IncorrectClassificationFeatureException("Integer parameters for ss_eval feature cannot be parsed correctly");
		}
		
		if(upstream_range<0 || downstream_range<0)
			throw new IncorrectClassificationFeatureException("Indicated ranges for ss_eval features are not permitted");
	}
	
	/**
	 * Because all classificationfeatures need different algorithms to draw them,
	 * according to the specifications, these utilities are also moved to the 
	 * implementations of the ClassificationFeatures. However, as not to clutter them,
	 * we have chosen to place this data in a seperate interface. The programmer thus
	 * has a choice of using one or two classes to implement both interfaces.
	 * 
	 * @return The object which can draw the classification features.
	 */
	public ClassificationFeatureDrawing getClassificationFeatureDrawing(){
		return new SecStructEvalClassificationFeatureDrawing(this);
	}
	
	
	public int getLength(){
		return 0;
	}
	
	/**
	 * Makes a copy of this particular classificationfeature. 
	 * The cloneable interface cannot be used directly because it is only applicable
	 * on classes, not interfaces.
	 * @return A copy of the classificationfeature
	 */
	public ClassificationFeature make_copy(){
		ClassificationFeature res = null;
		try{
			res = (ClassificationFeature)this.clone();
		}
		catch(Exception exc){}
		return res;
	}
}
