package util.classificationFeatures.implementations.test;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import util.classificationFeatures.AttributeDescription;
import util.classificationFeatures.ClassificationFeature;
import util.classificationFeatures.ClassificationFeatureDrawing;
import util.classificationFeatures.FeatureExtractionException;
import util.classificationFeatures.IncorrectClassificationFeatureException;
import util.conversions.Conversion;
import util.conversions.ConversionUtils;

public class VarianceClassificationFeature implements ClassificationFeature{

	
	private Conversion conversion 	= null;
	private int length 				= 0;
	private int upstream 			= 0;
	private int downstream 			= 0;
	

	/**
	 * This method extracts the features from a sequence (relative to the splicesite)
	 * and returns those features in a list of double-values (standardized on double
	 * values because of its generic but still small-sized nature).
	 * @param sequences The sequence from which the features should be extracted.
	 * @param splicesite The location of the splicesite (pseudo or real)
	 * @param types The compositional types (if requested, otherwise null).
	 * @param conversions A list with all the conversion types
	 * @param conversion_sequences  An array with all the converted sequences.
	 * @return A list with the features
	 * @throws FeatureExtractionException Thrown when something goes wrong when extracting the features.
	 */
	public List<Double> extractFeatures(String[] sequences, int splicesite, String[] types, List<Conversion> conversions, String[] conversion_sequences) throws FeatureExtractionException {
		
		/*
		 * Initialization of the variables
		 */
		List<Double> result = new ArrayList<Double>();
		String sequence = null;
		int conversion_index = conversions.indexOf(conversion);
		if(conversion_index!=-1)
			sequence = conversion_sequences[conversion_index];
		else
			sequence = sequences[0];
		if(sequence==null)
			throw new FeatureExtractionException("Stringsequence is null or could not be retrieved.");
		if(splicesite<0 || splicesite>sequence.length())
			throw new FeatureExtractionException("Splicesite is outside permitted range for positional feature -> splicesite : "+splicesite+" | sequence length : "+sequence.length());
		
		/*
		 * finding the correct start and stop sites
		 */
		int finalUpRange = 0;
		int finalDownRange = 0;
		boolean finalAddition = false;
		if(splicesite-upstream<0){
			finalUpRange = 0;
			for(int i=0;i<upstream-splicesite;i++){
				result.add(0.0);
				result.add(0.0);
				result.add(0.0);
			}
		}
		else
			finalUpRange = splicesite-upstream;
		if(splicesite+downstream>sequence.length()){
			finalDownRange = sequence.length();
			finalAddition = true;
		}
		else
			finalDownRange = splicesite+downstream;
		
		/*
		 * extract the features
		 */
		for(int i=finalUpRange;i<=finalDownRange-length;i++){
			String sub = sequence.substring(i,i+length);
			double[] res = findFeatures(sub);
			result.add(res[0]);
			result.add(res[1]);
			result.add(res[2]);
		}
		
		
		//if necessary, add empty types to end 
		if(finalAddition){
			for(int i=0;i<splicesite+downstream-finalDownRange;i++){
				result.add(0.0);
				result.add(0.0);
				result.add(0.0);
			}
		}
		
		return result;
	}
	
	//dream tales of dark tears
	private double[] findFeatures(String sub){
		double[] res = new double[3];
		int max_length = length/2;
		
		int current_length = 1;
		int current_number = 0;
		
		Set<String> set = new HashSet<String>();
		
		for(int i=1;i<=max_length;i++){
			for(int j=0;j<sub.length()-i;j++){
				String t = sub.substring(j,j+i);
				if(!set.contains(t)){
					set.add(t);
					int count = countOccurences(t,sub);
					if(count>1){
						if(i*count>current_length*current_number){
							current_length = i;
							current_number = count;
						}
					}
				}
			}
		}
		
		res[0] = current_length;
		res[1] = current_number;
		res[2] = sub.length()-current_length*current_number;
		return res;
	}

	public int countOccurences(String motif,String sequence){
		int result = 0;
		int motifLength = motif.length();
		for(int i=0;i<sequence.length()-motifLength;i++){
			String a = sequence.substring(i,i+motifLength);
			if(a.equals(motif))
				result++;
		}
		return result;
	}
	
	
	public ClassificationFeatureDrawing getClassificationFeatureDrawing() {
		return null;
	}


	public Conversion getConversion() {
		return conversion;
	}

	public int getLength() {
		return length;
	}

	public AttributeDescription getLocationDescription(int loc) {
		return null;
	}

	public int getNumberOfAttributes() {
		return 3*(upstream+downstream);
	}


	public int[] getUpDownRanges() {
		int[] res = {upstream,downstream};
		return res;
	}

	
	public ClassificationFeature make_copy() {
		ClassificationFeature res = null;
		try{
			res = (ClassificationFeature)this.clone();
		}
		catch(Exception exc){}
		return res;
	}

	public boolean needCompositionalTypes() {
		return false;
	}

	public void setParameters(String[] args) throws IncorrectClassificationFeatureException {
		if(!args[0].equals("VARIANCE")){
			throw new IncorrectClassificationFeatureException("Incorrect naming for variance feature"); 
		}
		try{
			length = Integer.parseInt(args[1]);
			upstream = Integer.parseInt(args[2]);
			downstream = Integer.parseInt(args[3]);
			String[] ct = new String[args.length-4];
			for(int i=4;i<args.length;i++)
				ct[i-4]=args[i];
			conversion = ConversionUtils.parseConversionFromString(ct);
		}
		catch(Exception exc){
			throw new IncorrectClassificationFeatureException("Incorrect parameters for variance feature");
		}
		
	}


	public void setUpDownRanges(int up, int down) {
		this.upstream = up;
		this.downstream = down;
	}


	public String toNoRangeString() {
		if(conversion==null)
			return "VARIANCE "+length;
		else
			return "VARIANCE "+length+" "+conversion.toString();
	}

	public String toStringUnderscore() {
		if(conversion==null)
			return "VARIANCE_"+length+"_"+upstream+"_"+downstream;
		else
			return "VARIANCE_"+length+"_"+upstream+"_"+downstream+"_"+conversion.toString();
	}
	
	public String toString(){
		if(conversion==null)
			return "VARIANCE "+length+" "+upstream+" "+downstream;
		else
			return "VARIANCE "+length+" "+upstream+" "+downstream+" "+conversion.toString();
	}

}
