/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.io;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import util.FastaFileData;
import util.secondaryStructure.SecondaryStructureData;

/**
 * Utility class for parsing and extracting data from files
 * @author Michiel Van Bel
 */
public class FileParser {
	
	
	public FileParser(){
		
	}
	
	/**
	 * Extracts the secondarystructure data from a file, according
	 * to the way it was written in the file by RNAfold.
	 * 
	 * Not only the secondary structure itself is extracted, the associated
	 * energies are also given, thus giving rise to more possibilities for 
	 * the secondary structure to be used as a starting point for feature
	 * extraction 
	 * @param fileName The name of the file
	 * @return Data object which contains a list with all the secondary structures, 
	 * and a list with all the associated energies.
	 * @throws IOException If the reading from the file fails for a reason.
	 *
	public SecondaryStructureData extractSecondaryStructuresFromFile(String fileName) throws IOException{
		List<String> data  = new ArrayList<String>();
		List<Double> energies = new ArrayList<Double>();
		File file = new File(fileName);
		if(!file.exists()){
			return null;
		}
		BufferedReader reader = new BufferedReader(new FileReader(file));
		String s = reader.readLine();
		while(s!=null){
			int length = s.length();
			String secstruct = reader.readLine();	//secondary structure file has 2 lines, only last is needed
			
			String temp = secstruct.substring(0,length);
			String energy = secstruct.substring(length,secstruct.length());
			energy = energy.substring(energy.indexOf("(")+1,energy.indexOf(")"));
			Double energ = Double.parseDouble(energy);
			
			energies.add(energ);
			data.add(temp);
			s = reader.readLine();//secondary structure file has 2 lines, only last is needed
		}
		
		SecondaryStructureData res = new SecondaryStructureData();
		res.setEnergies(energies);
		res.setStructures(data);
		return res;
	}*/
	
	
	public BufferedReader getReader(String fileName)throws IOException{
		if(fileName==null)
			return null;
		
		String fileName1 = fileName;
		String fileName2 = fileName.substring(1,fileName.length());
		
		BufferedReader reader = null;
		InputStream stream = null;
		
		stream = getClass().getResourceAsStream(fileName1);
		if(stream==null&&fileName.startsWith(".")){
			stream = getClass().getResourceAsStream(fileName2);
		}
		if(stream!=null){
			reader = new BufferedReader(new InputStreamReader(stream));
		}
		else{
			reader = new BufferedReader(new FileReader(new File(fileName1)));
		}
		return reader;
	}
	
	/**
	 * Extracts the secondarystructure data from a file, according
	 * to the way it was written in the file by RNAfold.
	 * 
	 * Not only the secondary structure itself is extracted, the associated
	 * energies are also given, thus giving rise to more possibilities for 
	 * the secondary structure to be used as a starting point for feature
	 * extraction 
	 * @param fileName The name of the file
	 * @return Data object which contains a list with all the secondary structures, 
	 * and a list with all the associated energies.
	 * @throws IOException If the reading from the file fails for a reason.
	 */
	public SecondaryStructureData extractSecondaryStructuresFromFile(String fileName) throws IOException{
		List<String> simple_structures = new ArrayList<String>();
		List<String> extra_structures = new ArrayList<String>();
		List<String> sequences = new ArrayList<String>();
		List<Double> simple_energies = new ArrayList<Double>();
		List<Double> extra_energies = new ArrayList<Double>();
		
		boolean extended_sec_struct = false;
		
		File file = new File(fileName);
		if(!file.exists())
			return null;
		
		//BufferedReader reader = new BufferedReader(new FileReader(file));
		BufferedReader reader = getReader(fileName);
		String s = reader.readLine(); //sequence
		/*
		 * first, determine whether the file contains the simple
		 * secondary structure data, or rather the extended secondary structure
		 * data.
		 */
		s = reader.readLine(); //normal secondary structure data (always was like this)
		s = reader.readLine();	//either this is a new sequence, or it is the extended secondary structure
		if(s.contains("."))//simple check -->extended Sec struct
			extended_sec_struct = true;
		else
			extended_sec_struct = false;
		reader.close();
		
		
		/*
		 * now, extract the actual secondary structure data
		 */
		//BufferedReader reader2 = new BufferedReader(new FileReader(file));
		BufferedReader reader2 = getReader(fileName);
		String t = reader2.readLine();
		
		while(t!=null){
			int length = t.length();
			String sequence = t ;	//is always the case
			String secstruct = reader2.readLine();	//secondary structure file has 2 lines, only last is needed
			String true_simpel_sec_struct = secstruct.substring(0,length);
			String energy = secstruct.substring(length,secstruct.length());
			energy = energy.substring(energy.indexOf("(")+1,energy.indexOf(")"));
			Double energ = Double.parseDouble(energy);
			
			sequences.add(sequence);
			simple_structures.add(true_simpel_sec_struct);
			simple_energies.add(energ);
			
			if(extended_sec_struct){
				secstruct = reader2.readLine();
				String true_extra_sec_struct = secstruct.substring(0,length);
				energy = secstruct.substring(length,secstruct.length());
				energy = energy.substring(energy.indexOf("[")+1,energy.indexOf("]"));
				Double extra_energ = Double.parseDouble(energy);
				
				extra_structures.add(true_extra_sec_struct);
				extra_energies.add(extra_energ);
			}
			t = reader2.readLine();
		}
		
		SecondaryStructureData res = new SecondaryStructureData();
		res.setEnergies(simple_energies);
		res.setExtra_energies(extra_energies);
		res.setExtended_structure(extended_sec_struct);
		res.setStructures(simple_structures);
		res.setExtra_structures(extra_structures);
		res.setSequences(sequences);
		
		return res;
	}
	
	
	
	/**
	 * Extracts a given amount of secondary structure data from a given file.
	 * 
	 * If the number of requested lines is close to the number of lines in the file,
	 * the random generator may have trouble generating the last correct and available 
	 * indices for the free slots.
	 * @param fileName The name of the file
	 * @param number The number of files to be extracted
	 * @return An object which contains the structures and the energies
	 * @throws IOException Thrown when reading the file goes wrong
	 * @throws Exception When number>lines in file 
	 */
	public SecondaryStructureData extractRandomSecondaryStructuresFromFile(String fileName, int number) throws IOException, Exception{
		List<String> structures = new ArrayList<String>();
		List<Double> energies = new ArrayList<Double>();
		
		//File file  = new File(fileName);
		//int nrOfLines = FileUtils.getLinesInFile(file);
		SecondaryStructureData temp = extractSecondaryStructuresFromFile(fileName);
		if(temp==null)
			return null;
		int nrOfLines = temp.getStructures().size();
		if(number>nrOfLines)
			throw new Exception("Requested #of lines exceeds limit of file");
		if(number<0)
			return temp;
		
		List<String> temp_structures = temp.getStructures();
		List<Double> temp_energies	= temp.getEnergies();
		boolean[] chosenLines = new boolean[nrOfLines];
		for(int i=0;i<nrOfLines;i++)
			chosenLines[i] = false;
		
		Random generator = new Random();
		int counter= 0 ;
		while(counter<number){
			int index = generator.nextInt(nrOfLines);
			if(!chosenLines[index]){
				structures.add(temp_structures.get(index));
				energies.add(temp_energies.get(index));
				chosenLines[index] = true;
				counter++;
			}
		}
		
		SecondaryStructureData result = new SecondaryStructureData();
		result.setEnergies(energies);
		result.setStructures(structures);
		
		//cleaning up memory overhead
		temp_structures.clear();
		temp_energies.clear();
		temp_structures = null;
		temp_energies = null;
		temp = null;
		
		return result;
	}
	
	/**
	 * Static method for extracting all DNA sequences (simple format, one 
	 * sequence on each line, those sequences should all have their
	 * (pseudo-) splicesites aligned at the same position (defined in
	 * main.Jaspr.SPLICESITE).
	 * @param fileName The name of the file
	 * @return A list of sequences.
	 * @throws IOException Thrown when the reading fails
	 */
	public List<String> extractSequencesFromFile(String fileName) throws IOException{
		List<String> data = new ArrayList<String>();
		//File file  =new File(fileName);
		//BufferedReader reader = new BufferedReader(new FileReader(file));
		BufferedReader reader = getReader(fileName);
		String s = reader.readLine();
		int counter = 0;
		while(s!=null){
			counter++;
			if(!s.contains("N")&& !s.startsWith(">")){
				String[] split = s.split("\t");
				data.add(split[0]);	//if there is rubbish at the end of the line, drop it
			}
			s = reader.readLine();
		}
		//System.out.println("File "+fileName+" contains "+counter+" lines ");
		return data;
	}
	
	
	/**
	 * This method extracts at random a certain number of unique lines from a file.
	 * If the number of requested lines is close to the total number of lines, there might
	 * be a problem as the random generator tries to generate the remaining few open slots.
	 * @param fileName The name of the file
	 * @param number The expected number of lines
	 * @return A list with the lines
	 * @throws IOException Possibly thrown when reading the file.
	 * 
	 */
	public List<String> extractRandomSequencesFromFile(String fileName, int number) throws IOException,Exception{
		List<String> data = new ArrayList<String>();
		//int linesInFile = FileUtils.getLinesInFile(new File(fileName));
		List<String> temp = extractSequencesFromFile(fileName);
		int linesInFile = temp.size();
		if(number>linesInFile)
			throw new Exception("File <"+fileName+"> contains less data than the indicated than the indicated extraction number : "+number);
		if(number<=0)
			return temp;
		boolean [] chosenLines = new boolean[linesInFile];
		for(int i=0;i<linesInFile;i++)
			chosenLines[i] = false;
		
		int counter = 0 ;
		Random generator = new Random();
		while(counter<number){
			int index = generator.nextInt(linesInFile);
			if(!chosenLines[index]){
				data.add(temp.get(index));
				chosenLines[index] = true;
				counter++;
			}
		}
		temp.clear();	//in order to preserve memory
		temp = null;
		return data;
	}
	
	
	/**
	 * Reads all the fasta data from a fastafile.
	 * This data is then stored in a FastFileData object in order to ensure
	 * maximum retention of fasta data.
	 * @param file The fastafile 
	 * @return Object which contains the parsed data from the fastafile
	 * @throws IOException Fileoperations are prone to errors
	 */
	public FastaFileData extractDataFromFastaFile(File file) throws IOException{
		FastaFileData data = new FastaFileData();
		if(file==null) return null; //dummy check
		//BufferedReader reader = new BufferedReader(new FileReader(file));
		BufferedReader reader = getReader(file.getAbsolutePath());
		String sequence = "";
		String name = null;
		String s = reader.readLine(); 
		if(!s.startsWith(">")){
			System.err.println("File "+file+" is not in fasta format");
			reader.close();
			return null;
		}
		if(s.indexOf('|')==-1) 
			name = s.substring(1,s.length());
		else
			name = s.substring(1,s.indexOf('|'));
		s = reader.readLine();
		while(s!=null){
			sequence+=s;
			s = reader.readLine();
		}
		
		data.setName(name);
		data.setSequence(sequence);
		reader.close();
		return data;
	}
	
	/**
	 * 
	 */
	public FastaFileData extractDataFromFastaEmblFile(File file) throws IOException{
		FastaFileData data = new FastaFileData();
		if(file==null) return null;
		//BufferedReader reader = new BufferedReader (new FileReader(file));
		BufferedReader reader = getReader(file.getAbsolutePath());
		String sequence = "";
		String name = null;
		String s = reader.readLine();
		s = reader.readLine();
		name = file.getName();
		s = reader.readLine();
		while(s!=null){
			sequence+=s;
			s = reader.readLine();
		}
		data.setName(name);
		data.setSequence(sequence);
		reader.close();
		return data;
	}
}
