/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;

import org.apache.log4j.Logger;

import base.ClassificationAction;


import util.RocCurveData;
import util.classificationFeatures.ClassificationFeature;
import util.results.CrossValidationResult;

/**
 * This class takes care of various I/O related tasks. 
 * @author Michiel Van Bel
 */
public class IOUtils {

	/*
	 * Some local variables
	 */
	private Logger logger = null;
	private ClassificationAction ca = null;
	
	
	/**
	 * Constructor. 
	 * @param logger Logging facility, really needed because a lot of methods
	 * might cause problems and throw exceptions.
	 * @param ca The classificationAction to which this instantiaton of IOUtils belongs
	 * to.
	 */
	public IOUtils(Logger logger, ClassificationAction ca){
		this.logger = logger;
		this.ca = ca;
	}
	
	
	/**
	 * Returns the number of lines (terminated by a carriage return) in the file
	 * @param file The file of which we want the number of lines
	 * @return The number of lines
	 */
	public int getLinesInFile(File file){
		int result = 0;
		try{
			BufferedReader reader = new BufferedReader(new FileReader(file));
			String s = reader.readLine();
			while(s!=null){
				result++;
				s = reader.readLine();
			}
			reader.close();
		}
		catch(Exception exc){
			logger.error(exc);
			return -1;
		}
		return result;
	}
	
	/**
	 * Makes a copy of the file (with a new name), and then destroys
	 * the original. This method can be used to apply featureselection
	 * to feature files.
	 * @param original The original File
	 * @return The copied file, with as name the original with "_copy" added
	 * @throws IOException When diskoperations/filesystemoperations fail 
	 */
	public File makeCopyAndDestroyOriginal(File original) throws IOException{
		File copy = null;
		BufferedReader reader = new BufferedReader(new FileReader(original));
		copy = new File(original.getPath()+"_copy");
		BufferedWriter writer = new BufferedWriter(new FileWriter(copy));
		String s = reader.readLine();
		while(s!=null){
			writer.write(s+"\n");
			s = reader.readLine();
		}
		reader.close();
		writer.close();
		original.delete();
		return copy;
	}
	
	/**
	 * Merges a number of files, by which it is assumed that the lines are space-formatted,
	 * and contain space seperated fields. The output is then sorted according to
	 * these numbers. Technically, the array of inputFiles should not contain
	 * the outputFile, but normally an exception is thrown when trying to write
	 * and read at the same time, so it should not cause problems.
	 * @param output The outputFile
	 * @param files The inputFiles
	 * @throws IOException Thrown when a diskoperation cannot be completed
	 * @throws NullPointerException Thrown when a file is null
	 */
	public static void mergeFiles(File output, File[] files) throws IOException, NullPointerException{
		int num = files.length;
		BufferedWriter writer = new BufferedWriter(new FileWriter(output));
		BufferedReader[] readers = new BufferedReader [num];
		for(int i=0;i<num;i++){
			readers[i] = new BufferedReader(new FileReader(files[i]));
		}
		String [] lines = new String[num];
		for(int i=0;i<num;i++){
			lines[i]=readers[i].readLine();
		}
		
		int lineTaken  = -1;
		boolean contin = !allNulls(lines);
		while(contin){
			lineTaken = getSmallest(lines);
			writer.write(lines[lineTaken]+"\n");
			lines[lineTaken] = readers[lineTaken].readLine();
			contin = !allNulls(lines);
		}
		writer.flush();
		for(int i=0;i<num;i++)
			readers[i].close();
		writer.close();
		for(int i=0;i<num;i++)
			files[i].delete();
			
		
	}
	
	/**
	 * Checks whether or not all the strings in an array (
	 * or the array itself) are null.
	 * @param lines The array with strings
	 * @return Are all string null?
	 */
	private static boolean allNulls(String[] lines){
		if(lines==null)
			return true;
		int nullCount = 0;
		for(int i=0;i<lines.length;i++)
			if(lines[i]==null)
				nullCount++;
		if(nullCount==lines.length)
			return true;
		else
			return false;
	}
	
	
	
	/**
	 * Returns the index of the string (in the array) that contains the smallest
	 * integer value 
	 * @param lines The array with (space formatted) strings
	 * @return The index of the string in the array with the smallest integer valeu
	 */
	private static int getSmallest(String[] lines)throws IOException{
		if(allNulls(lines))
			return -1;
		
		int [] max = new int[lines.length];
		for(int i=0;i<lines.length;i++)
			max[i] = Integer.MAX_VALUE;
		
		
		for(int i=0;i<lines.length;i++){
			if(lines[i]!=null){
				max[i] = Integer.parseInt(lines[i].split(" ")[0]);
			}
		}
		
		int minIndex = 0;
		int minimum = max[0];
		for(int i=0;i<max.length;i++){
			if(max[i]<minimum){
				minIndex = i;
				minimum = max[i];
			}
		}
		return minIndex;
	}
	
	
	
	/**
	 * This methods writes the results of a crossvalidation to a file. This includes 
	 * pretty basic stuff such as the amount of true positives and false positives, but also
	 * deducted information such as the F_1 measure and the specificity. 
	 * 
	 * While it may seem incoherent to need both the featurefile and trainPos/trainNeg data, 
	 * it is clear this is needed because we may use a special case of crossvalidation in 
	 * which the data is not split evenly, thus we need more information about the total
	 * amount of trainingdata
	 * 
	 * It is assumed that the crossvalidationresult has allready computed all the deducted
	 * information.
	 *
	 * @param result The crossvalidation result
	 * @param featureFile File containing the features
	 * @param trainPos The number positive training examples used
	 * @param testPos The number of positive test examples used 
	 * @param trainNeg The number of negative training examples used
	 * @param testNeg The number of negative test examples used
	 */
	public void writeCrossValidation(CrossValidationResult result, File featureFile
			,int trainPos, int testPos, int trainNeg, int testNeg){
		try{
			File output = null;
			output = new File(ca.getOutputdirectory()+ca.getClassification_name()+"_crossvalidation.txt");

			BufferedWriter writer = new BufferedWriter(new FileWriter(output,true));
			writer.write("TOTAL_POSITIVES : "+result.getTotalPositives()+"\t| TRUE_POSITIVES : "+result.getTruePositives()+"\t| FALSE_NEGATIVES : "+result.getFalseNegatives()+"\n");
			writer.write("TOTAL_NEGATIVES : "+result.getTotalNegatives()+"\t| TRUE_NEGATIVES : "+result.getTrueNegatives()+"\t| FALSE_POSITIVES : "+result.getFalsePositives()+"\n");
			writer.write("ERROR_RATE: "+result.getError_rate()+"\n");
			writer.write("ACCURACY: "+result.getAccuracy2()+"\n");
			writer.write("SENSITIVITY :"+result.getSensitivity()+"\n");
			writer.write("FP_RATE :"+result.getFp_rate()+"\n");
			writer.write("SPECIFICITY :"+result.getSpecificity()+"\n");
			writer.write("CORRELATION_COEFFICIENT :"+result.getCorrelation_coefficient()+"\n");
			writer.write("APPROXIMATE_CORRELATION :"+result.getApproximate_correlation()+"\n");
			writer.write("F1_MEASURE :"+result.getF1_measure()+"\n");
			writer.write("FP5%_MEASURE:"+result.getFp5_measure()+"\n");
			writer.write("AUC :"+result.getAuc()+"\n");
			writer.write("====================\n");
			writer.write("CLASSIFICATION FEATURES: \n");
			List<ClassificationFeature> list =ca.getClassificationFeatures();
			for(int i=0;i<list.size();i++){
				ClassificationFeature feat = list.get(i);
				writer.write("\t "+feat.toString()+"\n");
			}
			writer.write("====================\n");
			writer.write("DATA :\n");
			writer.write("\t TRAINING_EXAMPLES : "+getLinesInFile(featureFile)+"\t TOTAL_POSITIVE : "+(trainPos+testPos)+" \t TOTAL_NEGATIVE : "+(trainNeg+testNeg)+"\n");
			writer.write("\t POSITIVE_TRAINING_EXAMPLES : "+trainPos+"\n");
			writer.write("\t NEGATIVE_TRAINING_EXAMPLES : "+trainNeg+"\n");
			writer.write("\t POSITIVE_TEST_EXAMPLES : "+testPos+"\n");
			writer.write("\t NEGATIVE_TEST_EXAMPLES : "+testNeg+"\n");
			writer.write("====================\n");
			writer.write("====================\n");
			writer.write("\n\n\n");
			writer.close();	
		}
		catch(Exception exc){
			logger.error(exc);
		}
	}
	

	/**
	 * This method generates the Gnuplot-commands to display the roc-curves 
	 * (only possible with cross-validation). Two different gnuplot-command
	 * files are generated : 
	 * a) The first one displays all the roc-curves on a single display. While
	 * this works for a limited amount of roc-curves that are also rather divergent,
	 * it has limited applicability when the amount of roc-curves increases.
	 * b) A command which displays (and stores as png-files) all couples of roc-curves.
	 * This is thus easily usable for comparing all the roc-curves.
	 * @param fileNames A list with all the filenames and data which contain roc-curve data
	 * @param path The path in which to place the resulting gnuplot-command file
	 */
	public void writeRocGnuPlotCommand (List<RocCurveData> fileNames,String path){
		try{
			File f = new File(path+"roc_"+ca.getClassification_name()+".sh");
			
			BufferedWriter writer = new BufferedWriter(new FileWriter(f));
			writer.write("plot ");
			for(int i=0;i<fileNames.size();i++){
				if(i!=fileNames.size()-1)
					writer.write("'"+fileNames.get(i).getFileName()+"' with lines, ");
				else
					writer.write("'"+fileNames.get(i).getFileName()+"' with lines");
			}
			writer.write("\n");
			writer.flush();
			writer.close();
			
			
			File fd = new File(path+"roc_"+ca.getClassification_name()+"_dual.sh");
	
			BufferedWriter wr = new BufferedWriter(new FileWriter(fd));
			for(int i=0;i<fileNames.size();i++){
				for(int j=0;j<fileNames.size();j++){
					wr.write("set terminal png\n");
					wr.write("set output 'roc_output_"+ca.getClassification_name()+"_"+fileNames.get(i).getIdentifier()+"_"+fileNames.get(j).getIdentifier()+".png' \n");
					wr.write("set grid \n");
					wr.write("set xlabel \"FP-Rate\" \n");
					wr.write("set ylabel \"TP-Rate\" \n");
					
					wr.write("plot '"+fileNames.get(i).getFileName()+"' with lines, '"+fileNames.get(j).getFileName()+"' with lines \n");
					wr.write("\n");
				}
			}
			wr.flush();
			wr.close();
			
		}
		catch(Exception exc){
			logger.error(exc);
		}
	}
	
}
