/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.io;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.List;

import org.apache.log4j.Logger;

import util.Options;
import util.results.ClassificationResult;

public class OutputClassification {
	
	private Logger logger 	= null;
	private Options options = null;	
	
	public OutputClassification(Options options, Logger logger){
		this.options 	= options;
		this.logger		= logger;
	}
	
	/**
	 * This method writes the classification (of the possible splicesites in
	 *  a testfile) to an outputfile whose name is taken from the testfileheader.
	 *  
	 *  The confidence of the prediction is generated by going through all 
	 *  the classified distances and extracting the extremes, which are then
	 *  presented as "100%". 
	 *  
	 *  Possible splicesites are indicated by a '<--' arrow
	 *  Likeley splicesites (with the likely-cutoff being defined as a user
	 *  changeable variable) are indicated by a '<==' arrow
	 *
	 */
	public void writeClassification(List<ClassificationResult> classificationResults){
		if(classificationResults.size() == 0)
			return;
		try{
			// get the extremes, to calculate the confidences
			double maxPositiveDonor = 0.0;
			double maxNegativeDonor = 0.0;
			double maxPositiveAcceptor = 0.0;
			double maxNegativeAcceptor = 0.0;
			
			for(int i=0;i<classificationResults.size();i++){
				ClassificationResult res = classificationResults.get(i);
				double distance = res.getDistance();
				if(res.isDonor()){
					if(distance > maxPositiveDonor)	
						maxPositiveDonor = distance;
					if(distance < maxNegativeDonor)
						maxNegativeDonor = distance;
				}
				else{
					if(distance > maxPositiveAcceptor)	
						maxPositiveAcceptor = distance;
					if(distance < maxNegativeAcceptor)
						maxNegativeAcceptor = distance;	
				}
			}
			//write the classificationresults away
			File file = new File(options.OUTPUTDIRECTORY+classificationResults.get(0).getSeqName()+".out");
			BufferedWriter writer = new BufferedWriter(new FileWriter(file,true));
			for(int i=0;i<classificationResults.size();i++){
				ClassificationResult res = classificationResults.get(i);
				double distance =res.getDistance();
				double maxPositive = 0.0;
				double maxNegative = 0.0;
				if(res.isDonor()){
					writer.write("Donor\t");
					maxPositive = maxPositiveDonor;
					maxNegative = maxNegativeDonor;
				}
				else{
					writer.write("Acceptor\t");
					maxPositive = maxPositiveAcceptor;
					maxNegative = maxNegativeAcceptor;
				}
				writer.write("splicesite : "+res.getSplicesite()+"\t");
				writer.write("classification : "+res.getClassification()+"\t");
				double confidence = 0.0;
				if(distance > 0){
					if(maxPositive != 0.0)
						confidence = distance * 100 / maxPositive;
					else
						confidence = 0;
				}
				else{ 
					if(maxNegative != 0.0)
						confidence = distance * 100 / maxNegative;
					else
						confidence = 0;
				}
				
				confidence = (double)((int)(confidence*1000))/1000;
				distance = (double)((int)(distance*1000))/1000;
				
				//int tempConf = (int)confidence*100;
				
				writer.write("dist/maxDist*100 : " + confidence+"%\t\t");
				writer.write("distance : "+distance+"\t");
				
				if(res.getClassification().equals("1")){
					if(confidence>options.cutoffConfidence){
						writer.write("<==");
					}
					else{
						writer.write("<--");
					}
				}
				writer.write("\n");	
			}
			writer.close();
		}
		catch(Exception exc){
			logger.error(exc);
		}
	}
}
