/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.results;

import util.Util;

/*
 * last update : 11/04/2007
 * note : added some more comments
 */

/**
 * This class contains the normal results of a crossvalidation (number
 * of false positives/false negatives/true positives/true negatives and
 * the distances and labels of the results). From here on all other results
 * are extracted from this data. 
 * The memeory usage should not be a problem, since only 4 arrays of fixed size
 * are used (say 10000 training examples --> 4 * 8 * 10000 bytes = 320 kb).
 * 
 * @author Michiel Van Bel
 */

public class CrossValidationResult {

	//the basic results
	private int truePositives = 0;
	private int trueNegatives = 0;
	private int falsePositives = 0;
	private int falseNegatives = 0;
	private int totalPositives = 0;
	private int totalNegatives = 0;
	
	
	private double squaredCorrelationCoefficient = -1.0;
	private double meanSquaredError = -1.0;
	private double accuracy	= -1.0;
	
	private double error_rate = 0.0;
	private double accuracy2 = 0.0;
	private double sensitivity = 0.0;
	private double specificity = 0.0;
	private double fp_rate = 0.0;
	private double correlation_coefficient = 0.0;
	private double approximate_correlation = 0.0;
	private double f1_measure = 0.0;
	private double auc = 0.0;
	
	private double [] distances = null;
	private double [] labels = null;
	private double [] roc_tp_rate = new double[0];
	private double [] roc_fp_rate = new double[0];
	
	private double fp5_measure = 0.0;
	
	/**
	 * Creates a CrossValidationResult object. 
	 * No data values are initialized.
	 * All data should be set by the applicable setter-methods
	 */
	public CrossValidationResult(){
	}
	
	
	/**
	 * This method computes all the other results, following the 
	 * given basic data of the false positives/true positives/...
	 * 
	 * This includes for example correlation coefficient, f_1 measure, etc.
	 * This method should be called independently since there is no direct way of knowing when
	 * all the variables have been set.
	 */
	public void computeDeductedData(){
		int sum = truePositives+trueNegatives+falsePositives+falseNegatives;
		if(sum!=0){
			error_rate = ((double)(falsePositives+falseNegatives))/(double)sum;
			accuracy2 = (double)(1-error_rate);
		}
		if(truePositives+falseNegatives!=0)
			sensitivity = (double)(truePositives)/(double)(truePositives+falseNegatives);
		if(falsePositives+trueNegatives!=0)
			fp_rate = (double)falsePositives/(double)(falsePositives+trueNegatives);
		if(truePositives+falsePositives!=0)
			specificity = (double)(trueNegatives)/(double)(trueNegatives+falsePositives);
			//specificity = (double)truePositives/(double)(truePositives+falsePositives);
		
		int NN = trueNegatives+falseNegatives;
		int PP = truePositives+falsePositives;
		int PN = truePositives+falseNegatives;
		int NP = trueNegatives+falsePositives;
		int tot = NN*PP*NP*PN;
		if(tot!=0){
			double sub = Math.sqrt((double)tot);
			correlation_coefficient = ((double)truePositives*trueNegatives - falsePositives*falseNegatives)/sub;
		}
		try{
			double p1 = (double)truePositives/(double)(truePositives+falseNegatives);
			double p2 = (double)truePositives/(double)(truePositives+falsePositives);
			double p3 = (double)trueNegatives/(double)(trueNegatives+falsePositives);
			double p4 = (double)trueNegatives/(double)(trueNegatives+falseNegatives);
			approximate_correlation = 0.5*(p1+p2+p3+p4)-1;
		}
		catch(Exception exc){
			//do nothing
		}
		try{
			f1_measure = 2*specificity*sensitivity/(specificity+sensitivity);
		}
		catch(Exception exc){
			//do nothing
		}
		
	}
	
	/**
	 * computes the roc-curve data from the distances
	 * and labels that are provided.
	 */
	public void compute_roc(){
		//sort labels and distances
		Util.doubleSort(distances,labels);
		roc_tp_rate = new double[distances.length];
		roc_fp_rate = new double[distances.length];
		
		for(int i=0;i<distances.length;i++){
			int tp = 0;
			int fp = 0;
			int tn = 0;
			int fn = 0;
			for(int j=0;j<distances.length;j++){
				if(i>j){
					if(labels[j]<0)
						tn++;
					else
						fn++;
				}
				else{
					if(labels[j]<0)
						fp++;
					else
						tp++;
				}
			}
			//System.out.println("TP : "+tp+" | FN : "+fn+" | TN : "+tn+" | FP : "+fp);
			roc_tp_rate[distances.length-1-i]=(double)tp/(double)(tp+fn); //noemer is steeds echt aantal positieven
			roc_fp_rate[distances.length-1-i]=(double)fp/(double)(fp+tn); //noemer is steeds echt aantal negatieven
		}
		
		for(int i=0;i<roc_tp_rate.length-1;i++){
			double temp = ((roc_tp_rate[i]+roc_tp_rate[i+1])/2) * (roc_fp_rate[i+1]-roc_fp_rate[i]);
			auc+=temp;
		}
		
		double lastStop = 0.0;
		for(int i=0;i<roc_tp_rate.length;i++){
			double temp = roc_tp_rate[i];
			if(temp>lastStop&&temp<=0.95){
				lastStop = temp;
				fp5_measure = roc_fp_rate[i];
			}
		}
	}
	
	
	/**
	 * Returns the accuracy of the crossvalidation
	 * @return The accuracy of the crossvalidation
	 */
	public double getAccuracy() {
		return accuracy;
	}

	/**
	 * Returns the mean squared error of the crossvalidation
	 * @return The mean squared error of the crossvalidation
	 */
	public double getMeanSquaredError() {
		return meanSquaredError;
	}


	/**
	 * Returns the squared correlation coefficient of the crossvalidation
	 * @return The squared correlation coefficient of the crossvalidation
	 */
	public double getSquaredCorrelationCoefficient() {
		return squaredCorrelationCoefficient;
	}

	/**
	 * Returns the amount of false negatives of the crossvalidation
	 * @return The amount of false negatives of the crossvalidation
	 */
	public int getFalseNegatives() {
		return falseNegatives;
	}

	/**
	 * Sets the amount of false negatives of the crossvalidation
	 * @param falseNegatives The new amount of false negatives
	 */
	public void setFalseNegatives(int falseNegatives) {
		this.falseNegatives = falseNegatives;
	}

	/**
	 * Returns the amount of false positives of the crossvalidation
	 * @return The amount of false positives of the crossvalidation
	 */
	public int getFalsePositives() {
		return falsePositives;
	}

	
	/**
	 * Sets the amount of false positives
	 * @param falsePositives The new amount of false positives
	 */
	public void setFalsePositives(int falsePositives) {
		this.falsePositives = falsePositives;
	}

	/**
	 * Returns the amount of true negatives
	 * @return The amount of true negatives
	 */
	public int getTrueNegatives() {
		return trueNegatives;
	}

	/**
	 * Sets the amount of true negatives
	 * @param trueNegatives The new amount of true negatives
	 */
	public void setTrueNegatives(int trueNegatives) {
		this.trueNegatives = trueNegatives;
	}

	
	/**
	 * Returns the amount of true positives
	 * @return The amount of true positives
	 */
	public int getTruePositives() {
		return truePositives;
	}

	/**
	 * Sets the amount of true positives
	 * @param truePositives The new amount of true positives
	 */
	public void setTruePositives(int truePositives) {
		this.truePositives = truePositives;
	}

	/**
	 * Returns the amount of total negatives
	 * @return The total amount of negatives
	 */
	public int getTotalNegatives() {
		return totalNegatives;
	}

	/**
	 * Sets the total amount of negatives
	 * @param totalNegatives The total amount of negatives
	 */
	public void setTotalNegatives(int totalNegatives) {
		this.totalNegatives = totalNegatives;
	}

	/**
	 * Returns the total amount of positives
	 * @return The total amount of positves
	 */
	public int getTotalPositives() {
		return totalPositives;
	}

	/**
	 * Sets the total amount of positives
	 * @param totalPositives The new total amount of positives
	 */
	public void setTotalPositives(int totalPositives) {
		this.totalPositives = totalPositives;
	}

	/**
	 * Returns another accuracy measurement
	 * @return Accuracy measurement
	 */
	public double getAccuracy2() {
		return accuracy2;
	}

	/**
	 * Returns the approximate correlation
	 * @return The approximate correlation
	 */
	public double getApproximate_correlation() {
		return approximate_correlation;
	}

	/**
	 * Returns the correlation coefficient of the crossvalidation
	 * @return The correlation coefficient of the crossvalidation
	 */
	public double getCorrelation_coefficient() {
		return correlation_coefficient;
	}

	/**
	 * Returns the error rate of the crossvalidation
	 * @return The error rate of the crossvalidation
	 */
	public double getError_rate() {
		return error_rate;
	}

	/**
	 * Returns the sensitivity of the crossvalidation
	 * @return The sensitivity of the crossvalidation
	 */
	public double getSensitivity() {
		return sensitivity;
	}

	/**
	 * Returns the specificity of the crossvalidation
	 * @return The specificity of the crossvalidation
	 */
	public double getSpecificity() {
		return specificity;
	}

	/**
	 * Returns the false positive rate of the crossvalidation
	 * @return The false positive rate of the crossvalidation
	 */
	public double getFp_rate() {
		return fp_rate;
	}

	/**
	 * Returns the F1-measure of the crossvalidation (f1-measure
	 * is the harmonic means of sensitivity and specificity)
	 * @return The f1-measure of the crossvalidation
	 */
	public double getF1_measure() {
		return f1_measure;
	}


	/**
	 * Sets the distancesto the hyperplane for all training
	 * examples
	 * @param distances The distances of the crossvalidaded trainingexamples
	 */
	public void setDistances(double[] distances) {
		this.distances = distances;
	}

	/**
	 * Sets the labels (1/-1 for positive/negative) for all the
	 * training examples. These labels are the original known labels.
	 * @param labels The labels of the training examples.
	 */
	public void setLabels(double[] labels) {
		this.labels = labels;
	}


	/**
	 * Returns the false positive rate for the roc curve
	 * @return the false positive rate for the roc curve
	 */
	public double[] getRoc_fp_rate() {
		return roc_fp_rate;
	}


	/**
	 * Returns the true positive rate for the roc curve
	 * @return the true positive rate for the roc curve
	 */
	public double[] getRoc_tp_rate() {
		return roc_tp_rate;
	}

	/**
	 * Returns the FP5% measure from the crossvalidation
	 * @return The FP5% measure
	 */
	public double getFp5_measure() {
		return fp5_measure;
	}

	/**
	 * Returns the area under the roc-curve 
	 * (a measurement for the performance of a classifier).
	 * @return The area under the roc-curve
	 */
	public double getAuc() {
		return auc;
	}


	public double[] getDistances() {
		return distances;
	}


	public double[] getLabels() {
		return labels;
	}


	public void setAccuracy(double accuracy) {
		this.accuracy = accuracy;
	}


	public void setAccuracy2(double accuracy2) {
		this.accuracy2 = accuracy2;
	}


	public void setApproximate_correlation(double approximate_correlation) {
		this.approximate_correlation = approximate_correlation;
	}


	public void setAuc(double auc) {
		this.auc = auc;
	}


	public void setCorrelation_coefficient(double correlation_coefficient) {
		this.correlation_coefficient = correlation_coefficient;
	}


	public void setError_rate(double error_rate) {
		this.error_rate = error_rate;
	}


	public void setF1_measure(double f1_measure) {
		this.f1_measure = f1_measure;
	}


	public void setFp_rate(double fp_rate) {
		this.fp_rate = fp_rate;
	}


	public void setFp5_measure(double fp5_measure) {
		this.fp5_measure = fp5_measure;
	}


	public void setMeanSquaredError(double meanSquaredError) {
		this.meanSquaredError = meanSquaredError;
	}


	public void setRoc_fp_rate(double[] roc_fp_rate) {
		this.roc_fp_rate = roc_fp_rate;
	}


	public void setRoc_tp_rate(double[] roc_tp_rate) {
		this.roc_tp_rate = roc_tp_rate;
	}


	public void setSensitivity(double sensitivity) {
		this.sensitivity = sensitivity;
	}


	public void setSpecificity(double specificity) {
		this.specificity = specificity;
	}


	public void setSquaredCorrelationCoefficient(
			double squaredCorrelationCoefficient) {
		this.squaredCorrelationCoefficient = squaredCorrelationCoefficient;
	}

}
