package var;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import util.classificationFeatures.AttributeDescription;
import util.classificationFeatures.ClassificationFeature;
import util.classificationFeatures.ClassificationFeatureUtils;
import util.featureSelection.ValPosCombination;
import var.featureSelection.FeatureFileContent;

/**
 * This class is meant to find comparable results between featureselection filters of the full
 * type. Indeed, these filters can be written to a file and then compared, in case the same
 * features are used. An exact match of all the features is requested. While this is not 
 * strictly necessary (since we could be able to skip some of them and just compare the 
 * relevant classificationfeatures), it does make the program a lot more cleaner and 
 * easier to write. Thus, we have opted for this solution.
 * 
 * @author Michiel Van Bel
 *
 */
public class FSCompare {

	//File 1, file 2 : the feature file that should be compared 
	private String feature_filename_1 	= null;
	private String feature_filename_2 	= null;
	
	private String positive_training_file_1 = null;
	private String negative_training_file_1 = null;
	
	private String positive_training_file_2 = null;
	private String negative_training_file_2 = null;
	
	private int splicesite 					= 0;
	
	
	public static void main(String [] args){
		new FSCompare(args);
	}
	
	public FSCompare(String[] args){
		if(args.length!=1){
			System.out.println("Unable to start the application.");
			System.out.println("Usage : java FSCompare <config-file>");
			return;
		}
		parseParameterFile(args[0]);
		try{
			FeatureFileContent ffc_1 = extractContent(new File(feature_filename_1));
			FeatureFileContent ffc_2 = extractContent(new File(feature_filename_2));
			
			compareFeatureFiles(ffc_1,ffc_2);
		}
		catch(Exception exc){
			exc.printStackTrace();
			
		}
	}
	
	
	/**
	 * This method compares the different contents of the featurefiles.
	 * The method first tests whether or not the features are the same,
	 * and then goes on to actually test the content of these files if the 
	 * extracted and defined features are the same.
	 * Of course, if there has been manual tampering with the featurefiles in
	 * order to make it appear that the same classificationfeatures are used,
	 * the program could crash.
	 * @param ffc_1 Content of featurefile 1
	 * @param ffc_2 Content of featurefile 2
	 */
	public void compareFeatureFiles(FeatureFileContent ffc_1,FeatureFileContent ffc_2){
		boolean cfTest = testClassificationFeatures(ffc_1,ffc_2);
		if(!cfTest){
			System.out.println("The dedicated classification features are different between " +
					"both featurefiles");
			System.out.println("No comparison is possible");
			System.exit(1);
		}
		
		testSimilarLocations(ffc_1,ffc_2);
	}
	
	/**
	 * This method tests the same locations within both contents of featurefiles.
	 * It iterates over all possible features and checks their weights. If both the weights
	 * (of the same location of course) are not zero valued, this result is stored.
	 * Hereafter, the results are displayed with a possibe combination
	 * @param ffc_1 The content of feature file one
	 * @param ffc_2 The content of feature file two
	 */
	public void testSimilarLocations(FeatureFileContent ffc_1, FeatureFileContent ffc_2){
		List<ValPosCombination> list_1 = ffc_1.getVpcList();
		List<ValPosCombination> list_2 = ffc_2.getVpcList();
		
		List<Integer> same_list = new ArrayList<Integer>();
		List<Double> val_list_1 = new ArrayList<Double>();
		List<Double> val_list_2 = new ArrayList<Double>();
		
		int non_zero_1 = 0;
		int non_zero_2 = 0;
		
		
		for(int i=0;i<list_1.size();i++){
			ValPosCombination vpc1 = list_1.get(i);
			ValPosCombination vpc2 = list_2.get(i);
			double val1 = vpc1.getValue();
			double val2 = vpc2.getValue();
			if(val1!=0.0 && val1!=0)
				non_zero_1++;
			if(val2!=0.0 && val2!=0)
				non_zero_2++;
			
			if(val1!=0.0 && val1!=0 && val2!=0.0 && val2!=0){
				same_list.add(vpc1.getPosition());
				val_list_1.add(val1);
				val_list_2.add(val2);
			}
		}
		
		System.out.println("# Usable features "+ffc_1.getFileName()+" : "+non_zero_1);
		System.out.println("# Usable features "+ffc_2.getFileName()+" : "+non_zero_2);
		System.out.println("# corresponding features : "+same_list.size());
		
		//System.out.println(same_list.get(0)+" "+val_list_1.get(0)+" "+val_list_2.get(0));
		
		System.out.println("Location\t value1 \t value2 \t feature \t ");
		List<List<ValPosCombination>> attrib_list_1 = ClassificationFeatureUtils.divide(ffc_1.getCfList(), list_1);
		for(int i=0;i<same_list.size();i++){
			int loc = same_list.get(i);
			String explanation = explanation(loc,attrib_list_1,ffc_1.getCfList());
			System.out.println(loc+"\t"+val_list_1.get(i)+"\t"+val_list_2.get(i)+"\t"+explanation);
		}
	}
	

	
	/**
	 * Generates an explanation for the located feature. 
	 * It first determines to which classificationfeature the extracted feature belongs,
	 * and it then goes on determine what position the feature has within this classificationfeature.
	 * At last, with this location found
	 * @param loc Location in the total featurelist (can thus reach very high values)
	 * @param attrib_list Double nested list which determines the ranges of the classificationfeatures 
	 * @param cfList The list with used classificationfeatures
	 * @return String with an explanation
	 */
	public String explanation(int loc,List<List<ValPosCombination>> attrib_list,List<ClassificationFeature> cfList){
		String res = "";
		List<ValPosCombination> locatedList = null;
		int i =0;
		loc++;
		//find the correct classificationfeature (attrib_list.size() == cfList.size())
		//also selects the correct attributelist
		for(;i<attrib_list.size();i++){
			List<ValPosCombination> l_list = attrib_list.get(i);
			ValPosCombination vpc = l_list.get(l_list.size()-1);
			int pos = vpc.getPosition();
			if(loc<pos){
				locatedList = l_list;
				break;
			}
		}
		
		res+=cfList.get(i)+"\t";
		//System.out.println("size list : " + locatedList.size()+" | location : "+loc);
		//iterates over the attributelist
		int j =0;
		for(;j<locatedList.size();j++){
			ValPosCombination vpc = locatedList.get(j);
			int pos = vpc.getPosition();
			if(pos==loc){
				break;
			}
		}
		ClassificationFeature feat = cfList.get(i);
		AttributeDescription ad = feat.getLocationDescription(j);
		String temp = ad.toString();
		res+="\t"+temp;
		if(ad.findSupport){
			String support_1_p = checkSupport(ad,positive_training_file_1,splicesite,feat);
			String support_1_n = checkSupport(ad,negative_training_file_1,splicesite,feat);
			String support_2_p = checkSupport(ad,positive_training_file_2,splicesite,feat);
			String support_2_n = checkSupport(ad,negative_training_file_2,splicesite,feat);
			res+="\t"+support_1_p+"|"+support_1_n+"\t"+support_2_p+"|"+support_2_n;
		}
		
		return res;
	}
	
	private String checkSupport(AttributeDescription ad,String fileName,int splice,ClassificationFeature cf){
		String res = "";
		double result = 0;
		
		String[] types = ClassificationFeatureUtils.generateCompositionalTypes(cf.getLength());
		double [] support = new double[types.length];
		List<String> useList = new ArrayList<String>();
		
		for(int i=0;i<types.length;i++){
			useList.add(types[i]);
		}
		
		int[] topvalues = null;
		
		try{
			BufferedReader readerPos = new BufferedReader(new FileReader(new File(fileName)));
			String sequence = readerPos.readLine();
			double found = 0;
			double total = 0;
			int length = cf.getLength();
			while(sequence!=null){
				switch(ad.type){
				case POSITIONAL : 
					String t_p = "";
					if(ad.up_down.equals("upstream"))
						t_p = sequence.substring(splice-ad.location_bp,splice-ad.location_bp+length);
					else
						t_p = sequence.substring(splice+ad.location_bp,splice+ad.location_bp+length);
					
					int sd = useList.indexOf(t_p);
					if(sd!=-1)
						support[sd]++;
					if(t_p.equals(ad.value))
						found++;
					total++;
					
					break;
				case COMPOSITIONAL : 
					//String t_c = "";
					//if(ad.up_down.equals(arg0))
					
					break;
				case READING_FRAME : break;
				}
				
				sequence = readerPos.readLine();
			}
			if(total!=0){
				result = found/total;
				for(int i=0;i<support.length;i++){
					support[i]=support[i]/total;
				}
			}
			else
				result = total;
			
		}
		catch(Exception exc){
			exc.printStackTrace();
			System.exit(1);
		}
		res+=result;
		try{
			topvalues = getTopValueIndices(support,2);
			res+="["+types[topvalues[0]]+":"+support[topvalues[0]]+",";
			res+=types[topvalues[1]]+":"+support[topvalues[1]]+"]";
		}
		catch(Exception exc){
			System.out.println("Error finding top support : "+support.length+" "+topvalues[0]+" "+topvalues[1]);
		}
		return res;
	}
	
	private int[] getTopValueIndices(double[]array, int num){
		List<Double> tempList = new ArrayList<Double>();
		for(int i=0;i<array.length;i++)
			tempList.add(array[i]);
		int[] res = new int[num];
		
		for(int i=0;i<num;i++){
			double d = java.util.Collections.max(tempList);
			int index = tempList.indexOf(d);
			res[i] = index;
			tempList.set(index,new Double(0));
		}
		return res;
	}
	
	/**
	 * This method tests whether or not two featurefiles are generated by the 
	 * same amount and types of classificationfeatures.
	 * @param ffc_1 The content of feature file 1
	 * @param ffc_2 The content of feature file 2
	 * @return A boolean indicating whether or not both files are generated
	 * by the same classification features.
	 */
	public boolean testClassificationFeatures(FeatureFileContent ffc_1, FeatureFileContent ffc_2){
		List<ClassificationFeature> cfList1 = ffc_1.getCfList();
		List<ClassificationFeature> cfList2 = ffc_2.getCfList();
		
		if(cfList1.size()!=cfList2.size()){
			return false;
		}
		
		Set<String> cf_s_1 = new HashSet<String>();
		Set<String> cf_s_2 = new HashSet<String>();
		
		for(int i=0;i<cfList1.size();i++){
			String name1 = cfList1.get(i).toString();
			cf_s_1.add(name1);
			String name2 = cfList2.get(i).toString();
			cf_s_2.add(name2);
		}
		
		if(cf_s_1.equals(cf_s_2))
			return true;
		else
			return false;
	}
	
	/**
	 * This method extracts the content of a featurefile, parses it and
	 * then goes on to store it all in one object.
	 * @param file The featurefile
	 * @return An object with the parsed content of the file:
	 *  a list with the different defined features and a list with all the locations and their weights 
	 * @throws IOException In case of a file reading exception
	 */
	public FeatureFileContent extractContent(File file) throws IOException{
		if(!file.exists()){
			System.err.println("featurefile "+file.getPath()+" does not exist");
			System.exit(1);
		}
		FeatureFileContent ffc = new FeatureFileContent();
		ffc.setFileName(file.getName());
		List<ClassificationFeature> cfList = new ArrayList<ClassificationFeature>();
		List<ValPosCombination> vpcList = new ArrayList<ValPosCombination>();
		
		BufferedReader reader = new BufferedReader(new FileReader(file));
		String s = reader.readLine();
		ffc.setClassification_name(s);
		s = reader.readLine();
		
		while(s!=null){
			
			if(s.startsWith("CLASSIFICATION_FEATURE")){
				s = s.substring(s.indexOf("\t")+1,s.length());
				ClassificationFeature cf = ClassificationFeatureUtils.parseFeatureFromString(s);
				cfList.add(cf);
			}
			else{
				String [] split = s.split("\t");
				double value = Double.parseDouble(split[0]);
				int position = Integer.parseInt(split[1]);
				if(position==0&&value==1.0){} //classattrib
				else{
					ValPosCombination vpc = new ValPosCombination(value,position);
					vpcList.add(vpc);
				}
			}
			s = reader.readLine();
		}
		reader.close();
		
		ffc.setCfList(cfList);
		ffc.setVpcList(vpcList);		
		return ffc;
	}
	
	private void parseParameterFile(String fileName){
		File file = new File(fileName);
		if(!file.exists()||file.isDirectory()){
			System.out.println("configuration file does not exist");
			System.exit(1);
		}
		try{
			BufferedReader reader = new BufferedReader(new FileReader(file));
			String s = reader.readLine();
			while(s.startsWith("#"))
				s = reader.readLine();
			String [] split = s.split("\t+");
			if(split[0].equals("FEATURE_FILENAME_1"))
				feature_filename_1 = split[1]; 
			else
				exitFailure(" no FEATURE_FILENAME_1 defined");
			
			s = reader.readLine();
			split = s.split("\t+");
			if(split[0].equals("POSITIVE_TRAINING_FILE_1"))
				positive_training_file_1 = split[1];
			else
				exitFailure(" no POSITIVE_TRAINING_FILE_1 defined");
			
			s = reader.readLine();
			split = s.split("\t+");
			if(split[0].equals("NEGATIVE_TRAINING_FILE_1"))
				negative_training_file_1 = split[1];
			else
				exitFailure(" no NEGATIVE_TRAINING_FILE_1 defined");
			
			s = reader.readLine();
			split = s.split("\t+");
			if(split[0].equals("FEATURE_FILENAME_2"))
				feature_filename_2 = split[1];
			else
				exitFailure(" no FEATURE_FILENAME_2 defined");
			
			s = reader.readLine();
			split = s.split("\t+");
			if(split[0].equals("POSITIVE_TRAINING_FILE_2"))
				positive_training_file_2 = split[1];
			else
				exitFailure("no POSITIVE_TRAINING_FILE_2 defined");
			
			s = reader.readLine();
			split = s.split("\t+");
			if(split[0].equals("NEGATIVE_TRAINING_FILE_2"))
				negative_training_file_2 = split[1];
			else
				exitFailure("no NEGATIVE_TRAINING_FILE_2 defined ");
			
			s = reader.readLine();
			split = s.split("\t+");
			if(split[0].equals("SPLICESITE"))
				splicesite = Integer.parseInt(split[1]);
		}
		catch(Exception exc){
			exc.printStackTrace();
			exitFailure("General failure");
		}
	}
	
	private void exitFailure(String s){
		System.err.println(s);
		System.exit(1);
	}
}
